<?php

/**
 * Checked Itentities
 *
 * Plugin sends a confirmation message to new identity
 *
 * @version 2.8.5 - 18.10.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 **/
 
/**
 *
 * Usage: http://mail4us.net/myroundcube/
 *
 **/ 
 
class checked_identities extends rcube_plugin
{
  public $task = 'login|settings|dummy';
  public $noajax = true;
  
  /* unified plugin properties */
  static private $plugin = 'checked_identities';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = '<a href="http://myroundcube.com/myroundcube-plugins/checked-identities-plugin" target=_new>Documentation</a>';
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '2.8.5';
  static private $date = '18-10-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.8.1',
    'PHP' => '5.2.1'
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';
  
  function init()
  {
    $this->add_texts('localization/', false);
    $this->add_hook('startup', array($this, 'enableidentity'));
    $this->register_action('plugin.deleteidentity', array($this, 'deleteidentity'));
    $this->add_hook('identity_create', array($this, 'checkidentity'));
    $this->add_hook('template_object_identityform', array($this, 'disableemailfield'));
    $this->add_hook('template_object_identitieslist', array($this, 'waitingforconfirmation'));
    $this->add_hook('preferences_sections_list', array($this, 'prefssection'));
    $this->add_hook('preferences_list', array($this, 'prefs_dummy'));
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
    }
  }
  
  static public function about($keys = false){
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $plugin,
            );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function prefssection($args)
  {
    $skin = rcmail::get_instance()->config->get('skin');
    if($skin == 'classic' || $skin == 'litecube-f' || $skin == 'groupvice4' || $skin == 'meh'){
      $args['list']['identitieslink']['id'] = 'identitieslink';
      $args['list']['identitieslink']['section'] = $this->gettext('identities');
    }
    return $args;
  }

  function prefs_dummy($args)
  {
    if ($args['section'] == 'identitieslink') {
      $args['blocks']['main']['options']['identitieslink']['title']    = $this->gettext('identities') . " ::: " . $_SESSION['username'];
      $args['blocks']['main']['options']['identitieslink']['content']  = "<script type='text/javascript'>\n";
      $args['blocks']['main']['options']['identitieslink']['content'] .= "  parent.location.href='./?_task=settings&_action=identities'\n";
      $args['blocks']['main']['options']['identitieslink']['content'] .= "</script>\n";
    }

    return $args;

  }
 
  function waitingforconfirmation($p)
  {
    $rcmail = rcmail::get_instance();

    $sql_result = $rcmail->db->query("
            SELECT * FROM ".get_table_name('identities')."
            WHERE  user_id=?
            AND    standard=?",
            $rcmail->user->data['user_id'],
            '-1');
    
    $waitings = array();        
    while($IDENTITIES = $rcmail->db->fetch_assoc($sql_result))
      $waitings[] = $IDENTITIES;

    if(isset($waitings[0]['identity_id'])){

      $out = "";
      // find me: how to detect odd/even ending of the default list?
      $class = "even";
      $i = 0;
      $skin = $rcmail->config->get('skin');
      if(!file_exists('./skins/' . $skin . '/images/icons/delete.png'))
        $skin = "classic";
        
      foreach($waitings as $key => $identity){

        $out .= '<tr class="contact ' . $class . '"><td title="' . $this->gettext('waintingforapproval','checked_identities') . '" class="name"><img alt="' . $this->gettext('delete') . '" title="' . $this->gettext('delete') . '" onclick="document.location.href=\'./?_task=settings&_action=plugin.deleteidentity&_iid=' . $identity['identity_id'] . '\'" src="./skins/' . $skin . '/images/icons/delete.png" align="absmiddle" />&nbsp;' . $identity['name'] . "\n";
        $out .= '<span title="' . $this->gettext('waintingforapproval','checked_identities') . '" class="email">&lt;' . $identity['email'] . '&gt;</span>' . "\n";
        $out .= '</tr>' . "\n";
        
        if($i==0){
          $i=1;
          $class = "odd";
        }
        else{
          $i=0;
          $class = "even";
        }
      }

      $out .= '</tbody>' . "\n";
      $out = str_replace("</tbody>",$out,$p['content']);

      $p['content'] = $out;

    }    

    return $p;

  }
  
  function deleteidentity()
  {
    $iid = get_input_value('_iid', RCUBE_INPUT_GET);

    $rcmail = rcmail::get_instance();
    if($iid)
      $sql_result = $rcmail->db->query("
            DELETE FROM ".get_table_name('identities')."
            WHERE  identity_id=?",
            $iid);

    $rcmail->output->redirect(array('_task' => 'settings', '_action' => 'identities'));
    exit;
  }
  
  function disableemailfield($p)
  {
     $p['content'] .='
  <!-- begin plugin checked identities -->
  <script type="text/javascript"><!--
    try{
      // set readonly attribute if field is not empty (edit mode)
      if(document.forms.form._email.value != "")
        document.forms.form._email.readOnly="readonly";
      else
        document.forms.form._standard.disabled="disabled";
    }
    catch(e){
      // Todo add some kind of redirect on failure here later ...
    }
  
  //-->
  </script>
  <!-- end plugin checked identities -->
  ';
    return $p;;
  }

  function enableidentity()
  {
    $rcmail = rcmail::get_instance();
    if($rcmail->action == 'plugin.enableidentity'){
      // do the query to set del = 0
      $email = trim($_GET['email']);
      $name  = urldecode(trim($_GET['name']));
      $user_id = trim($_GET['user_id']);
      $token = trim($_GET['token']);
      if($token == md5($user_id . $email . urlencode($name))){
        $query = "UPDATE ".get_table_name('identities')."
                  SET  del=0, standard=0 
                  WHERE user_id=?
                  AND email=? 
                  AND name=?";
        $ret = $rcmail->db->query($query,$user_id,$email,$name);
       
        if($ret){
          $res = $rcmail->db->affected_rows($ret);
          if($res > 0){
            $type = "confirmation";
            $message = $this->gettext('identityactive');
          }
          else{
            $type = "notice";
            $message = $this->gettext('checked_identities.isactive');
          }
        }
        else{
          $type = "error";
          $message = $this->gettext('errorsaving');
        }
      }
      else{
        $type = "error";
        $message = $this->gettext('errorsaving');
      }
      $rcmail->output->command('display_message', $message, $type);
      if($_SESSION['user_id'] == get_input_value('user_id', RCUBE_INPUT_GET)){
        $rcmail->output->add_script('window.setTimeout("self.close()", 8000)', 'docready');
      }
      else{
        $rcmail->output->add_script('window.setTimeout("document.location.href=' . "'" . './' . "'" . '", 8000);', 'docready');
      }
      $rcmail->output->send('checked_identities.success');
      exit;
    }
  }
  
  function checkidentity($a)
  {
    $record = $a['record'];
    if (!check_email($record['email'])){
      $a['abort'] = true;
      return $a;
    }
    
    $rcmail = rcmail::get_instance();
    
    if(strtolower($record['email']) == strtolower($rcmail->user->data['username']))
      return $a;
      
    if(class_exists("hmail_login",false)){
      $address = hmail_login::resolve_alias($record['email']);
      if(strtolower($address) == strtolower($rcmail->user->data['username']))
        return $a;
    }
        
    $user_id = $rcmail->user->data['user_id'];
    
    $rcmail->db->query("
            SELECT * FROM ".get_table_name('identities')."
            WHERE  name=?
            AND    email=?
            AND    user_id=?",
            trim($record['name']),
            strtolower($record['email']),            
            $user_id);
            
    $IDENTITIES = $rcmail->db->fetch_assoc();
    $rc = true;
    if(!is_array($IDENTITIES)){
      // add identity
      $a_cols = array(
        0 => 'email',
        1 => 'name',
        2 => 'organization',
        3 => 'reply-to',
        4 => 'bcc',
        5 => 'signature',
        6 => 'html_signature',
        7 => 'del',
        8 => 'standard'
      );
      
      foreach($a_cols as $key => $col)
         $arr[] = $rcmail->db->quoteIdentifier($col);					
      $a_cols = $arr;

      $a_values = array(
        0 => "'" . strtolower(trim($record['email'])) . "'",
        1 => "'" . trim($record['name']) . "'",
        2 => "'" . trim($record['organization']) . "'",
        3 => "'" . trim($record['reply-to']) . "'",
        4 => "'" . trim($record['bcc']) . "'",
        5 => "'" . trim($record['signature']) . "'",
        6 => "'" . trim($record['html_signature']) . "'",					
        7 => "'1'",
        8 => "'-1'",
      );

      $rc=$rcmail->db->query("
            INSERT INTO ".get_table_name('identities')."
             (user_id, ".join(', ', $a_cols).")
             VALUES (?, ".join(', ', $a_values).")",
             $user_id
          );
            
      $rcmail->db->query("
        DELETE FROM ".get_table_name('identities')."
         WHERE del=? AND standard<>?",
         1,
         -1
       );              
            
    }
   	   
    if($rc){
      $a = $this->sendmessage($a);
      if($a['success']){
        $type = "confirmation";
        $message = sprintf($this->gettext('checkaccount'), $record['email']);
      }
      else{
        $type = "error";
        $message = $this->gettext('sendingfailed');
        $rcmail->db->query("
                DELETE FROM ".get_table_name('identities')."
                    WHERE	user_id='" . $user_id . "'
                    AND email=" . $a_values[0] . "
                    AND name=" . $a_values[1] . "
                    AND del=" . $a_values[2] . "
                    AND standard=" . $a_values[3]
                    );       
      }
      $rcmail->output->command('display_message', $message, $type);
      if($rcmail->config->get('skin', 'classic') == 'larry'){
        $rcmail->output->add_script("setTimeout(\"parent.location.href='./?_task=settings&_action=identities'\",5000);");
        $rcmail->output->send('iframe');
      }
      else{
        $rcmail->output->include_script('list.js');
        $rcmail->output->add_script("setTimeout(\"rcmail.enable_command('add',true)\",1000);");
        $rcmail->output->send('identities');
      }
      exit;
    }
    
    $a['abort'] = true;
    return $a;
  
  }
  
  function sendmessage($a)
  {
    $record = $a['record'];
    $rcmail = rcmail::get_instance();
    $user_id = $rcmail->user->data['user_id'];
    $email = trim($record['email']);
    $name  = urlencode(trim($record['name']));

    $s = "";
    if(isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == TRUE)
      $s = "s";
    $roundcube_url = "http$s://" . $_SERVER['HTTP_HOST'] . $_SERVER['PHP_SELF'];
    
    $token = md5($user_id . $email . $name);
    $link  = '<a target="_new" href="' . $roundcube_url . '?_task=dummy&_action=plugin.enableidentity&user_id=' . $user_id . '&email=' . $email . '&name=' . $name . '&token=' . $token . '">'; 
    $link .= rcube_label("here","checked_identities");
    $link .= '</a>';

    if(file_exists("./plugins/checked_identities/localization/" . $_SESSION['language'] . ".inc")){
      $lang = $_SESSION['language'];
    }
    else{
      $lang = "en_US";
    }
  
    if(file_exists("./plugins/checked_identities/localization/templates/" . $lang . "/send_identity_subject.txt")){
      $subject_pattern = file_get_contents("./plugins/checked_identities/localization/templates/" . $lang . "/send_identity_subject.txt");
    }
    else if(file_exists("./plugins/checked_identities/localization/templates/en_US/send_identity_subject.txt")){
      $subject_pattern = file_get_contents("./plugins/checked_identities/localization/templates/en_US/send_identity_subject.txt");
    }
    else{
      $subject_pattern = file_get_contents("./plugins/checked_identities/localization/templates/en_US/send_identity_subject.txt.dist");
    }
       
    if(file_exists("./plugins/checked_identities/localization/templates/" . $lang . "/send_identity_body.html")){
      $body_pattern = file_get_contents("./plugins/checked_identities/localization/templates/" . $lang . "/send_identity_body.html");
    }
    else if(file_exists("./plugins/checked_identities/localization/templates/en_US/send_identity_body.html")){
      $body_pattern = file_get_contents("./plugins/checked_identities/localization/templates/en_US/send_identity_body.html");
    }
    else{
      $body_pattern = file_get_contents("./plugins/checked_identities/localization/templates/en_US/send_identity_body.html.dist");
    }
    
    $footer = $rcmail->config->get('checked_identities_footer');

    if($body_pattern && $subject_pattern){
    
      $subject = sprintf(
                      $subject_pattern,
                      urldecode($name) . " (" . $email . ")"
                      );
                      
      if (function_exists('mb_encode_mimeheader')){
        mb_internal_encoding(RCMAIL_CHARSET);
        $subject= mb_encode_mimeheader($subject,
          RCMAIL_CHARSET, 'Q', $rcmail->config->header_delimiter(), 8);
      }
      else{
        $subject = '=?UTF-8?B?'.base64_encode($subject). '?=';
      }                      
                    
      $body = sprintf(
                      $body_pattern,
                      $rcmail->config->get('default_font', 'Verdana, Geneva, sans-serif'),
                      urldecode($name) . " (" . $email . ")",
                      $link,
                      $footer,
                      $_SESSION['username'],
                      $_SESSION['username'],
                      $rcmail->config->get('admin_email')
                      );
                    
      $body = str_replace("\r","",$body);
      $body = str_replace("\n","\r\n",$body);
      if($rcmail->config->get('bounce_is_safe')){
        $from = $_SESSION['username'] . $rcmail->config->get('username_domain');
        if (!preg_match("/$this->EMAIL_ADDRESS_PATTERN/i", $from)){
          $from = $rcmail->config->get('admin_email');
        }
      }
      else{
        $from = $rcmail->config->get('admin_email');
      }
      
      $to = $email;
      if(!strstr($to,'@'))
        $to = $to . '@' . $rcmail->config->get('mail_domain');
      
      $mpb = md5(rand() . microtime());
      $headers  = "Return-Path: $from\r\n";
      $headers .= "MIME-Version: 1.0\r\n";
      $headers .= "Content-Type: multipart/alternative; boundary=\"=_$mpb\"\r\n";
      $headers .= "Date: " . date('r', time()) . "\r\n";
      $headers .= "From: $from\r\n";
      $headers .= "To: $to\r\n";
      $headers .= "Subject: $subject\r\n";
      $headers .= "Reply-To: $from\r\n";

      $txt_body  = "--=_$mpb";
      $txt_body .= "\r\n";
      $txt_body .= "Content-Transfer-Encoding: 7bit\r\n";
      $txt_body .= 'Content-Type: text/plain; charset="' . RCMAIL_CHARSET . '"' . "\r\n";
      $LINE_LENGTH = $rcmail->config->get('line_length', 72);
      $h2t = new html2text($body, false, true, 0);
      $txt = rc_wordwrap($h2t->get_text(), $LINE_LENGTH, "\r\n");
      $txt = wordwrap($txt, 998, "\r\n", true);
      $txt_body .= "$txt\r\n";
      $txt_body .= "--=_$mpb";
      $txt_body .= "\r\n";
          
      $msg_body .= $txt_body;
          
      $msg_body .= "Content-Transfer-Encoding: quoted-printable\r\n";
      $msg_body .= 'Content-Type: text/html; charset="' . RCMAIL_CHARSET . '"' . "\r\n\r\n";
      $msg_body .= str_replace("=","=3D",$body);
      $msg_body .= "\r\n\r\n";
      $msg_body .= "--=_$mpb--";
      $msg_body .= "\r\n\r\n";

      // send message
      if($rcmail->config->get('default_smtp_server')=="") { //PHP Mail Function 
        $rc = mail($to, $subject, $msg_body, $headers);
      }
      else {
        if(!is_object($rcmail->smtp))
          $rcmail->smtp_init(true);
        $rcmail->smtp->connect();
        $rc = $rcmail->smtp->send_mail($from, $to, $headers, $msg_body);
      } 
    
      if($rc){
        $a['success'] = true;
      }

    }
    return $a;
  }
}
?>