<?php

/**
 * DNS Blacklist
 *
 * Plugin sets $_SESSION['dnsblacklisted'] = [list host ... e.g. 19.240.20.217.bl.spamcop.net] if client IP is blacklisted
 *        or   $_SESSION['dnsblacklisted'] = 'pass' if IP check passed
 *        and  $_SESSION['clientip'] in any case.
 *
 * @version 2.1 - 10.09.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 **/
 
/**
 * Usage: http://mail4us.net/myroundcube/
 *
 **/ 
 
class dnsbl extends rcube_plugin
{
  public $noframe = true;
  public $noajax = true;
  
  private $ip; // ='203.176.130.6'; // Bad IP Address;
  
  /* unified plugin properties */
  static private $plugin = 'dnsbl';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = 'Re-configuration required';
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '2.1';
  static private $date = '10-09-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.7.1',
    'PHP' => '5.2.1'
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';

  function init() {
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
    }
    $this->add_hook('startup', array($this, 'dnsbl_do'));
    // block at least sending messages ::: $rcmail_config['dnsblacklists_logout'] = FALSE;
    $this->add_hook('message_outgoing_headers', array($this, 'dnsbl_block_sending'));
  
  }
  
  static public function about($keys = false){
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $plugin,
            );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function dnsbl_do($args) {
    if($this->dnsbl_check()){
      $rcmail = rcmail::get_instance();
      if($rcmail->config->get('dnsblacklists_logout',true)){
        $args['task'] = 'logout';
        $this->add_texts('localization/');
        $rcmail->output->show_message('dnsbl.dnsblacklisted','error');
        if(method_exists($rcmail->output,'add_script')){
          foreach($rcmail->config->get('dnsblacklists_remove', array()) as $val){
            $rcmail->output->add_script('$("' . $val . '").remove();','docready');
          }
        }
      }
    }
    return $args;
  }
  
  function dnsbl_block_sending($args){  
    if($_SESSION['dnsblacklisted'] != 'pass'){
      $this->add_texts('localization/');
      $rcmail = rcmail::get_instance();
      $rcmail->output->show_message('dnsbl.dnsblacklisted','error');
      $rcmail->output->send('iframe');
    }
    return $args;
  }
  
  function dnsbl_check(){
    if($this->ip == ""){
      $this->ip = $this->getVisitorIP();
    }
    if($this->ip == ""){
      return false;
    }
    if(empty($_SESSION['dnsblacklisted'])){
      // set flag for a whole browser session !!!
      $_SESSION['clientip'] = $this->ip;
      if($res = $this->dnsbl_blacklisted($this->ip)){
        $_SESSION['dnsblacklisted'] = $res;
        if($this->ip)
          $this->log(sprintf("--> Positive DNSBL result detected for IP %s", $this->ip));
        return true;
      }
      else{
        $_SESSION['dnsblacklisted'] = 'pass';
      }
    }
    if($_SESSION['dnsblacklisted'] != 'pass')
      return true;
    
    return false;
  }  
  
  function getVisitorIP() {
     global $HTTP_SERVER_VARS;
    //Regular expression pattern for a valid IP address 
    $ip_regexp = "/^([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})/"; 

    //Retrieve IP address from which the user is viewing the current page 
    if (isset ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"]) && !empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) { 
      $visitorIP = (!empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) ? $HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"] : ((!empty ($HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'])) ? $HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'] : @ getenv ('HTTP_X_FORWARDED_FOR')); 
    } 
    else { 
      $visitorIP = (!empty ($HTTP_SERVER_VARS['REMOTE_ADDR'])) ? $HTTP_SERVER_VARS['REMOTE_ADDR'] : ((!empty ($HTTP_ENV_VARS['REMOTE_ADDR'])) ? $HTTP_ENV_VARS['REMOTE_ADDR'] : @ getenv ('REMOTE_ADDR')); 
    }
    if(!$visitorIP)
      $visitorIP = "detection failed"; 
    return $visitorIP; 
  }
  
  function dnsbl_blacklisted($ip) {
    $rcmail = rcmail::get_instance();
    $dnsbl_lists = $rcmail->config->get('dnsblacklists', array());
    $reverse_ip = implode(".", array_reverse(explode(".", $ip)));
    foreach ($dnsbl_lists as $dnsbl_list => $val){
      if($val != 0){
        $prefix = '';
        if($val != 1){
          $prefix = $val . ".";
        }
        //trailing dot needed to prevent search domain additions.
        $lookup = $prefix . $reverse_ip . "." . $dnsbl_list . ".";
        $recs = @dns_get_record($lookup, DNS_A);
        if(is_array($recs)){
          break;
        }
      }
    }
    if(is_array($recs) && count($recs) > 0){
      $this->log("DNSBL: " . $reverse_ip . "." . $dnsbl_list);
      return $reverse_ip . "." . $dnsbl_list;
    }
    else
      return false;
  }
  
  function log($log){ 
    $rcmail = rcmail::get_instance();
    if($rcmail->config->get('dnsblacklists_log',true)){
      write_log('dnsbl',$log);
    }  
  }
}

?>