<?php

/**
 * google_analytics
 *
 * Bind google analytics script
 *
 * @version 1.0 - 16.09.2012
 * @author Roland 'rosali' Liebl
 * @modified_by Ondra 'Kepi' Kudlík
 * @website http://github.com/igloonet/roundcube_google_analytics
 * @licence GNU GPL
 *
 **/

class google_analytics extends rcube_plugin
{
  public $noajax = true;
  public $noframe = true;

  /* unified plugin properties */
  static private $plugin = 'google_analytics';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = null;
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '1.0';
  static private $date = '16-09-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.8.1',
    'PHP' => '5.2.1'
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';

  function init(){
    $this->add_hook('render_page', array($this, 'add_script'));
  }
  
  static public function about($keys = false){
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
             $requirements[$prefix.'plugins'][$plugin] = array(
               'method' => $method,
               'plugin' => $plugin,
             );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }


  function add_script($args){
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
    }
    $include = array_flip($rcmail->config->get('google_analytics_include', array()));
    if(isset($include[$args['template']])){
       if($rcmail->config->get('google_analytics_privacy')){
        if($_SESSION['user_id']){
          return $args;
        }
      }

      $google_analytics_domain = $rcmail->config->get('google_analytics_domain');
      $set_domain_name = '';

      // async mode 
      if($rcmail->config->get('google_analytics_async')){
        // set domain if not empty
        if ( !Empty($google_analytics_domain) )
          $set_domain_name = "  _gaq.push(['_setDomainName', '" . $rcmail->config->get('google_analytics_domain') . "']);";

        $script = "
<script type=\"text/javascript\">
  var _gaq = _gaq || [];
  _gaq.push(['_setAccount', '" . $rcmail->config->get('google_analytics_id') . "']);
  _gaq.push(['_setDomainName', '" . $rcmail->config->get('google_analytics_domain') . "']);
  _gaq.push(['_trackPageview']);
</script>
";
        // async mode add script to head of page
        $rcmail->output->add_header($script);

        $script = "
<script type=\"text/javascript\">     
  (function() {
    var ga = document.createElement('script'); ga.type = 'text/javascript'; ga.async = true;
    ga.src = ('https:' == document.location.protocol ? 'https://ssl' : 'http://www') + '.google-analytics.com/ga.js';
    var s = document.getElementsByTagName('script')[0]; s.parentNode.insertBefore(ga, s);
  })();
</script>
";
        $rcmail->output->add_footer($script);

      // sync mode
      } else {
        // set domain if not empty
        if ( !Empty($google_analytics_domain) )
          $set_domain_name = 'pageTracker._setDomainName("' . $rcmail->config->get('google_analytics_domain') . '");';

        $script = '
<script type="text/javascript">
var gaJsHost = (("https:" == document.location.protocol) ? "https://ssl." : "http://www.");
document.write(unescape("%3Cscript src=\'" + gaJsHost + "google-analytics.com/ga.js\' type=\'text/javascript\'%3E%3C/script%3E"));
</script>
<script type="text/javascript">
try {
var pageTracker = _gat._getTracker("' . $rcmail->config->get('google_analytics_id') . '");
' . $set_domain_name . '
pageTracker._trackPageview();
} catch(err) {}</script>
    ';
    
        // sync mode add script to end of page
        $rcmail->output->add_footer($script);
      }
    }
    return $args;
  }
}

?>
