<?php

/**
 * Change hMailServer Forwarding
 *
 * Plugin that gives access to Forwarding using hMailServer database
 *
 * @version 2.4 - 29.10.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 * @TODO Perhaps a date picker and expiration time ...
 *
 **/
 
/**
 * Usage: http://mail4us.net/myroundcube/
 *
 *
 * REQUIREMENTS: (*) hMailServer 5.x
 *               (*) settings plugin @website http://myroundcube.googlecode.com 
 *                   (since Roundcube 0.6 the plugin is loaded automatically)
 *
 **/ 

class hmail_forwarding extends rcube_plugin
{
  public $task = 'mail|settings';
  
  private $sql_select = 'SELECT * FROM hm_accounts WHERE accountaddress=%u';
  private $sql_update = 'UPDATE hm_accounts SET accountlastlogontime=%t, accountforwardenabled=%o, accountforwardaddress=%a, accountforwardkeeporiginal=%k WHERE accountaddress=%u';

  /* unified plugin properties */
  static private $plugin = 'hmail_forwarding';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = 'Requires hMailserver.';
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '2.4';
  static private $date = '29-10-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.7.1',
    'PHP' => '5.2.1',
    'required_plugins' => array('settings' => 'require_plugin'),
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';

  function init()
  {
    $rcmail = rcmail::get_instance();
    if($rcmail->user->data['username'] == $_SESSION['username']){  
      $this->_load_config();
      $this->add_texts('localization/');
      if($rcmail->task == 'mail'){
        $this->add_hook('template_object_summary_forwarding', array($this, 'hmail_forwarding_form_obj'));
      }
      else{
        $this->register_action('plugin.hmail_forwarding', array($this, 'hmail_forwarding_init'));
        $this->register_action('plugin.hmail_forwarding-save', array($this, 'hmail_forwarding_save'));
        $this->register_action('plugin.hmail_forwarding-uninstall', array($this, 'hmail_forwarding_uninstall'));
        $this->register_handler('plugin.hmail_forwarding_form', array($this, 'hmail_forwarding_form'));
        $this->register_action('plugin.hmail_forwarding_show', array($this, 'frame'));
        $this->include_script('hmail_forwarding.js');
      }
    }
  }
  
  static public function about($keys = false)
  {
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
             $requirements[$prefix.'plugins'][$plugin] = array(
               'method' => $method,
               'plugin' => $plugin,
             );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function _load_config()
  {
    $rcmail = rcmail::get_instance();    
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
      $this->require_plugin('settings');
    }
  }
   
  function hmail_forwarding_init()
  {

    $rcmail = rcmail::get_instance();
    $rcmail->output->set_pagetitle($this->gettext('forwarding')); 
    $rcmail->output->send('hmail_forwarding.hmail_forwarding');
    
  }
  
  function frame()
  {
    $rcmail = rcmail::get_instance();
    $rcmail->output->add_script("rcmail.add_onload(\"rcmail.sections_list.select('accountlink')\");");
    $rcmail->output->add_script("rcmail.add_onload(\"document.frames['prefs-frame'].location.href='./?_task=settings&_action=plugin.hmail_forwarding&_framed=1'\");");       
    $rcmail->output->send("settings");
    exit;
  }  
  
  function hmail_forwarding_save()
  {
    
    $rcmail = rcmail::get_instance();
    if(strtolower($this->get_demo($_SESSION['username'])) == strtolower(sprintf($rcmail->config->get('demo_user_account'),""))){
      $rcmail->output->command('display_message', $this->gettext('demoaccount'), 'error');
      $this->hmail_forwarding_init();
    }  
    $user = strtolower($rcmail->user->data['username']);
    
    $enabled = (NULL != get_input_value('_forwardingenabled', RCUBE_INPUT_POST));
    if(!$enabled)
      $enabled = 0;
    if(isset($_SESSION['dnsblacklisted']) && $_SESSION['dnsblacklisted'] != 'pass')
      $enabled = 0;      
    $keepcopies   = get_input_value('_keepcopies', RCUBE_INPUT_POST);
    if(!$keepcopies)
      $keepcopies = 0;    
    $address      = strtolower(get_input_value('_forwardingaddress', RCUBE_INPUT_POST));

    if(class_exists("hmail_login",false)){
      $address = hmail_login::resolve_alias($address);
    }
  
    if($address == $user){
      $rcmail->output->command('display_message', $this->gettext('hmail_forwarding.forwardingloop'), 'error');
    }
    else if (!check_email($address)) {
      $rcmail->output->command('display_message', $this->gettext('hmail_forwarding.invalidaddress'), 'error');
    }
    else{
      if (!($res = $this->_save($user,$enabled,$keepcopies,$address))) {
        if(isset($_SESSION['dnsblacklisted']) && $_SESSION['dnsblacklisted'] != 'pass'){
          $rcmail->output->command('display_message',sprintf(rcube_label('dnsblacklisted', 'hmail_forwarding'),$_SESSION['clientip']),'error');        
        }
        else{
          $rcmail->output->command('display_message', $this->gettext('successfullysaved'), 'confirmation');
        }
      }
    } 

    $this->hmail_forwarding_init();
  
  }
  
  function hmail_forwarding_form_obj($args)
  {
    $rcmail = rcmail::get_instance();  
    $settings = $this->_get();
    
    $enabled = $settings['accountforwardenabled'];
    $address = $settings['accountforwardaddress'];
    
    if($enabled == 1){
      $class = "warning";
      $label_stat = $this->gettext('forwardingisenabled');
      $label_stat .= '&nbsp;...&nbsp;(' . $address . ')';
    }
    else{
      $class = "ok";
      $label_stat = $this->gettext('forwardingdisabled');      
    }            
    
    $out  = '<fieldset><legend>&nbsp;' . $this->gettext('forwarding') . '&nbsp;</legend>';
    $out .= '<br /><div>';
    $out .= '&nbsp;' . $label_stat . '&nbsp;...';
    $out .= $label_sens;    
    $out .= '&nbsp;[<a class= "summary' . $class . ' summaryaction" href="./?_task=settings&_action=plugin.hmail_forwarding_show">' . $this->gettext('edit') . '</a>]';
    $out .= '</div><br />';
    $out .= '</fieldset>';
    
    $args['content'] = $out;
    return $args;
  }  

  function hmail_forwarding_form()
  {
    $rcmail = rcmail::get_instance();

    // add some labels to client
    $rcmail->output->add_label(
      'hmail_forwarding.forwarding',    
      'hmail_forwarding.invalidaddress',
      'hmail_forwarding.forwardingloop'
    );
    
    $rcmail->output->add_script("var settings_account=true;");  

    $settings = $this->_get();
    
    $enabled     = $settings['accountforwardenabled'];
    $address     = $settings['accountforwardaddress'];
    $keepcopies  = $settings['accountforwardkeeporiginal'];
          
    $rcmail->output->set_env('product_name', $rcmail->config->get('product_name'));
    $rcmail->output->set_env('framed', true);

    // return the complete edit form as table
    $user = $rcmail->user->data['username'];
    if($_SESSION['global_alias'])
      $user = $_SESSION['global_alias'];
    $out .= '<fieldset><legend>' . $this->gettext('forwarding') . ' ::: ' . $user . '</legend>' . "\n";
    $out .= '<br />' . "\n";
    $out .= "<table>\n";

    // show forwarding properties  
    $field_id = 'forwardingenabled';
    $input_forwardingenabled = new html_checkbox(array('name' => '_forwardingenabled', 'id' => $field_id, 'value' => 1));

    $out .= sprintf("<tr><td class=\"title\"><label for=\"%s\">%s</label>:</td><td>%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('forwardingenabled')),
                $input_forwardingenabled->show($enabled?1:0));
                
    $field_id = 'forwardingaddress';
    $input_forwardingaddress = new html_inputfield(array('name' => '_forwardingaddress', 'id' => $field_id, 'value' => $address, 'maxlength' => 320, 'size' => 40));

    $out .= sprintf("<tr><td class=\"title\"><label for=\"%s\">%s</label>:</td><td>%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('forwardingaddress')),
                $input_forwardingaddress->show($address));
                
    $field_id = 'keepcopies';
    $input_keepcopies = new html_checkbox(array('name' => '_keepcopies', 'id' => $field_id, 'value' => 1));

    $out .= sprintf("<tr><td class=\"title\"><label for=\"%s\">%s</label>:</td><td>%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('keepcopies')),
                $input_keepcopies->show($keepcopies?1:0));                                                

    $out .= "\n</table>";
    $out .= '<br />' . "\n";
    $out .= "</fieldset>\n";    

    $rcmail->output->add_gui_object('forwardingform', 'forwarding-form');
    
    return $out;
  }
 
  private function _get()
  {
    $rcmail = rcmail::get_instance();
      
    $sql = $this->sql_select;

    if ($dsn = $rcmail->config->get('db_hmail_forwarding_dsn')) {
      // #1486067: enable new_link option
      if (is_array($dsn) && empty($dsn['new_link']))
        $dsn['new_link'] = true;
      else if (!is_array($dsn) && !preg_match('/\?new_link=true/', $dsn))
        $dsn .= '?new_link=true';    
      $db = new rcube_mdb2($dsn, '', FALSE);
      $db->set_debug((bool)$rcmail->config->get('sql_debug'));
      $db->db_connect('r');
    } else {
      die("FATAL ERROR ::: RoundCube Plugin ::: hmail_forwarding ::: \$rcmail_config['db_hmail_forwarding_dsn'] undefined !!! ==> die");
    }
    if ($err = $db->is_error())
      return $err;
      
    $sql = str_replace('%u', $db->quote($rcmail->user->data['username'],'text'), $sql);
    $res = $db->query($sql);
                 
    if ($err = $db->is_error()){
       return $err;
    }
    $ret = $db->fetch_assoc($res);

    return $ret;  
  }
  
  function hmail_forwarding_uninstall()
  {
    $rcmail = rcmail::get_instance();
    $user        = $rcmail->user->data['username'];
    $enabled     = 0;
    $keepcopies  = 0;
    $body        = '';
    $this->_save($user,$enabled,$keepcopies,$address);
    $rcmail->output->command('plugin.plugin_manager_success', '');
  }
  
  private function _save($user,$enabled,$keepcopies,$address)
  {
    $cfg = rcmail::get_instance()->config;
    
    if ($dsn = $cfg->get('db_hmail_forwarding_dsn')) {
      // #1486067: enable new_link option
      if (is_array($dsn) && empty($dsn['new_link']))
        $dsn['new_link'] = true;
      else if (!is_array($dsn) && !preg_match('/\?new_link=true/', $dsn))
        $dsn .= '?new_link=true';    
      $db = new rcube_mdb2($dsn, '', FALSE);
      $db->set_debug((bool)$cfg->get('sql_debug'));
      $db->db_connect('w');
    } else {
      die("FATAL ERROR ::: RoundCube Plugin ::: hmail_forwarding ::: \$rcmail_config['db_hmail_forwarding_dsn'] undefined !!! ==> die");
    }
    if ($err = $db->is_error())
      return $err;
    $sql = $this->sql_update;
    $sql = str_replace('%t',  $db->quote(date ('Y-m-d H:i:s', time()),'text'), $sql);    
    $sql = str_replace('%a',  $db->quote($address,'text'), $sql);            
    $sql = str_replace('%o',  $db->quote($enabled,'text'), $sql);
    $sql = str_replace('%k',  $db->quote($keepcopies,'text'), $sql);
    $sql = str_replace('%u',  $db->quote($user,'text'), $sql);
     
    $res = $db->query($sql);
    
    $this->reload($user);
    
    if ($err = $db->is_error())
      return $err;
    $res = $db->affected_rows($res);
    if ($res == 0) return $this->gettext('errorsaving');
    if ($res == 1) return FALSE; // THis is the good case - 1 row updated
    return $this->gettext('internalerror');

  }
  
  private function get_demo($string)
  {
    $temparr = explode("@",$string);
    return preg_replace ('/[0-9 ]/i', '', $temparr[0]) . "@" . $temparr[count($temparr)-1];   
  }
  
  private function reload($user)
  {
    $rcmail = rcmail::get_instance();
    if($rcmail->config->get('hmail_client')){
      $temparr = explode("@",$user);
      $domain = $temparr[1];
      if(!class_exists('Http')){
        include INSTALL_PATH . 'plugins/hmail_forwarding/class.http.php';
      }
      $httpConfig['method']     = 'POST';
      $httpConfig['target']     = $rcmail->config->get('hmail_client') . '?_action=reload';
      $httpConfig['timeout']    = '30';
      $httpConfig['params']     = array('_domain' => $domain, '_username' => $user, '_oldpw' => $rcmail->decrypt($_SESSION['password']));
      $http = new Http();
      $http->initialize($httpConfig);
      if(ini_get('safe_mode') || ini_get('open_basedir')){
        $http->useCurl(false);
      }
      $http->execute();
      if($http->error){
        write_log('errors', 'Plugin hmail_forwarding: Connection failed [' . $rcmail->config->get('hmail_client') . '?_action=reload]');
        return false;
      }
      else{
        $response = $http->result;
        if($response == 'ok'){
          return true;
        }
        else{
          write_log('errors', 'Plugin hmail_forwarding: Reload request failed [' . $user . ']');
          return false;
        }
      }
    }
  }
}

?>