<?php

/**
 * hMailServer Login
 *
 * Login Alias as Real User
 *
 * @version 1.7.2 - 10.09.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 **/
 
/**
 * Usage: http://mail4us.net/myroundcube/
 *
 * REQUIREMENTS: (*) hMailServer 5.x
 *
 **/ 

 
class hmail_login extends rcube_plugin
{
  public $task = 'login';
  public $noframe = true;
  public $noajax = true;
  
  private $sql_select_daalias = "SELECT * FROM hm_domain_aliases WHERE daalias=%d;";
  private $sql_select_domain  = "SELECT * FROM hm_domains WHERE domainid=%id;";
  private $sql_select_name    = "SELECT * FROM hm_aliases WHERE aliasname=%u;";
  
  /* unified plugin properties */
  static private $plugin = 'hmail_login';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = 'Requires hMailserver.';
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '1.7.2';
  static private $date = '10-09-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.7.1',
    'PHP' => '5.2.1',
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';

  public function init()
  { 
    $this->_load_config('hmail_login');
    $this->add_hook('startup', array($this, 'modify_user'));
    $this->add_hook('login_after', array($this, 'add_identity'));
  }
  
  static public function about($keys = false)
  {
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $plugin,
            );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }

  function _load_config($plugin)
  {
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
    }
  }  

  function modify_user($args)
  {
    if(isset($_POST['_user']) && isset($_POST['_pass'])){
      $rcmail = rcmail::get_instance();
      $user = get_input_value('_user', RCUBE_INPUT_POST);
           
      $temp = explode("@",$user);
      if(count($temp) == 1 && $user != ""){
        $user = get_input_value('_user', RCUBE_INPUT_POST) . "@" . $rcmail->config->get('hmail_default_domain');
      }

      $_POST['_logon_user'] = $user;
      
      $user = $this->resolve_username($user);
      $_POST['_user'] = $user;
        
    }
   
    return $args;
  }
  
  function resolve_username($user)
  {
    $temp = explode("@",$user);
    if(count($temp == 2)){
      $domain = $this->resolve_domain($temp[1]);
      $user   = $this->resolve_name($temp[0],$domain);    
    }

    return strtolower($user);
       
  }
  
  function resolve_name($name,$domain){
  
    if($name == "")
      return "";

    $rcmail = rcmail::get_instance();
    
    if ($dsn = $rcmail->config->get('db_hmail_login_dsn')) {
      // #1486067: enable new_link option
      if (is_array($dsn) && empty($dsn['new_link']))
        $dsn['new_link'] = true;
      else if (!is_array($dsn) && !preg_match('/\?new_link=true/', $dsn))
        $dsn .= '?new_link=true';    
      $db = new rcube_mdb2($dsn, '', FALSE);
      $db->set_debug((bool)$rcmail->config->get('sql_debug'));
      $db->db_connect('r');
    } else {
      die("FATAL ERROR ::: RoundCube Plugin ::: hmail_login ::: \$rcmail_config['db_hmail_login_dsn'] undefined !!! ==> die");
    }
    if ($err = $db->is_error())
      return $name; 
    if($this->sql_select_name == "")
      $this->sql_select_name = "SELECT * FROM hm_aliases WHERE aliasname=%u;";             
    $sql = str_replace('%u', $db->quote(strtolower($name . "@" . $domain),'text'), $this->sql_select_name);    
    $res = $db->query($sql);
      
    if ($err = $db->is_error()){
      return $name;
    }
    
    $ret = $db->fetch_assoc($res);   

    if(isset($ret['aliasvalue'])){
      $name = $ret['aliasvalue'];
    }
    else{
      $name = $name . "@" . $domain;
    }
   
    return $name;
  
  }
  
  function resolve_domain($domain)
  {
  
    $rcmail = rcmail::get_instance();
    
    if ($dsn = $rcmail->config->get('db_hmail_login_dsn')) {
      // #1486067: enable new_link option
      if (is_array($dsn) && empty($dsn['new_link']))
        $dsn['new_link'] = true;
      else if (!is_array($dsn) && !preg_match('/\?new_link=true/', $dsn))
        $dsn .= '?new_link=true';    
      $db = new rcube_mdb2($dsn, '', FALSE);
      $db->set_debug((bool)$rcmail->config->get('sql_debug'));
      $db->db_connect('r');
    } else {
      die("FATAL ERROR ::: RoundCube Plugin ::: hmail_login ::: \$rcmail_config['db_hmail_login_dsn'] undefined !!! ==> die");
    }
    if ($err = $db->is_error())
      return $domain;    
    if($this->sql_select_daalias == "")
      $this->sql_select_daalias = "SELECT * FROM hm_domain_aliases WHERE daalias=%d;";
    $sql = str_replace('%d', $db->quote(strtolower($domain),'text'), $this->sql_select_daalias);
         
    $res = $db->query($sql);
    
    if ($err = $db->is_error()){
       return $domain;
    }
    
    $ret = $db->fetch_assoc($res);    

    if(is_array($ret)){
      if($this->sql_select_domain == "")
        $this->sql_select_domain = "SELECT * FROM hm_domains WHERE domainid=%id;"; 
      $sql = str_replace('%id', $db->quote( $ret['dadomainid'],'text'), $this->sql_select_domain);
       
      $res = $db->query($sql);
    
      if ($err = $db->is_error()){
        return $domain;
      }
    
      $ret = $db->fetch_assoc($res);    
 
      if(isset($ret['domainname']))
        $domain = $ret['domainname'];
      
    }

    return $domain;
  
  }
  
  function resolve_alias($user)
  {
    $temp = explode("@",$user);
    $name = $temp[0];
    if(count($temp == 2)){
      $domain = $temp[1];
      $domain = hmail_login::resolve_domain($domain);
      $user = hmail_login::resolve_name($name,$domain);
    }
    
    return $user;
      
  }
  
  function add_identity($a)
  {
 
    $rcmail = rcmail::get_instance();
    
    if($_SESSION['impersonate'])
      return $a;
    
    if(!empty($_POST['_logon_user'])){
      if(strtolower(trim($_POST['_logon_user'])) != strtolower($rcmail->user->data['username'])){
        $alias = strtolower(trim($_POST['_logon_user']));
        // sanitiy check
        if (check_email($alias))
          $_SESSION['username_alias'] = $alias;
        else
          return $a;
      }
      else
        return $a;
    }
    else
      return $a;
              
    // get this user's identities (incl. deteted)
    $sql_result = $this->list_identities_history(); 

    if ($rcmail->db->num_rows($sql_result))
    {
      while ($sql_arr = $rcmail->db->fetch_assoc($sql_result))
      {
        if(strtolower($sql_arr['email']) == $alias)
          return $a;
        else
          $identities[] = $sql_arr;
      }
    
    }
        
    $temp = explode("@",$alias);
    $name = ucwords($temp[0]) . " (" . $temp[1] . ")";
    
    $rcmail->user->insert_identity(array('name' => $name, 'email' => $alias));
    
    return $a;
    
  }

  function list_identities_history($sql_add = '')
  {
  
    $rcmail = rcmail::get_instance();
    
    $id = $rcmail->user->ID;
     
    // get contacts from DB
    $sql_result = $rcmail->db->query(
      "SELECT * FROM ".get_table_name('identities')."
       WHERE  user_id=?
       $sql_add
       ORDER BY ".$rcmail->db->quoteIdentifier('standard')." DESC, name ASC, identity_id ASC",
      $id);
    
    return $sql_result;
  }

}  

?>