<?php

/**
 * Language Selector on Logon Page
 *
 * Switch language from Logon Page and adjust user language
 *  if language selector was touched
 *  else don't update users's language preference !
 *
 *
 * @version 2.2 - 10.09.2012
 * @author Roland Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 */
 
/**
 *
 * Usage: http://mail4us.net/myroundcube/
 *
 * Requirements: settings plugin
 *
 **/ 
 
class lang_sel extends rcube_plugin
{
  public $task = 'login|logout|settings';
  public $noajax = true;
  
  /* unified plugin properties */
  static private $plugin = 'lang_sel';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = null;
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '2.2';
  static private $date = '10-09-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.7.1',
    'PHP' => '5.2.1',
    'required_plugins' => array('settings' => 'require_plugin'),
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';

  function init(){
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->require_plugin('settings');
      $this->load_config();
    }
    $this->add_hook('template_object_loginform', array($this, 'show_selector'));
    $this->add_hook('startup', array($this, 'switch_lang_before_login'));
    $this->add_hook('login_after', array($this, 'switch_lang_after_login'));
    
  }
  
  static public function about($keys = false){
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $plugin,
            );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function show_selector($p){
  
    $rcmail = rcmail::get_instance();
    $limit_langs = array_flip((array)$rcmail->config->get('limit_languages', array()));
    $avail_langs = $rcmail->list_languages();
    if(count($limit_langs) > 0){
      foreach($limit_langs as $key => $val)
        $limit_langs[$key] = $avail_langs[$key];
      $langs = $limit_langs;
    }
    else
      $langs = $avail_langs;
    $style = '';
    if($rcmail->config->get('skin') == 'larry'){
      $style = "border-radius: 3px; -moz-border-radius: 3px; -webkit-border-radius: 3px; margin-left: 8px;";
    }
    $selector = '<select style="' . $style . '" onchange="document.location.href=\'./?_action=plugin.lang_sel&amp;_lang_sel=\' + this.value" name="_lang_sel">' . "\n";
    asort($langs);
    foreach($langs as $key => $val){
      if($_SESSION['language'] == $key)
        $selected = " selected=\"selected\"";
      else
        $selected = "";
      $selector .= '<option value="' . $key . "\"$selected>" . $val . "</option>\n";
    }
    $selector .= "</select>\n";

    $content = $p['content'];
    
    $content = str_replace("</tbody>","",$content);
    $content = str_replace("</table>","",$content);
    $content .= '<tr><td class="title"><label for="rcmlangsel">' . rcube_label('language') . '</label></td>' . "\n";
    $content .= '<td>' . $selector . '</td></tr>' . "\n";
    $content .= "</tbody>\n";
    $content .= "</table>\n";
   
    $p['content'] = $content;
        
    return $p;
  
  }
  
  function switch_lang_before_login($args){
    if(get_input_value('_lang_sel', RCUBE_INPUT_GET) && count($_POST) == 0){
      if(get_input_value('_action', RCUBE_INPUT_GET) == "plugin.lang_sel"){
        $rcmail = rcmail::get_instance();
        if(isset($_SESSION['username'])){
          $rcmail->output->redirect(array());
        }
        else{
          $lang = get_input_value('_lang_sel', RCUBE_INPUT_GET);
          $add = $this->add_plugin_localization($lang);
          $rcmail->load_language($lang,$add);
          $rcmail->output->send("login");
        }
      }
    }
    else{
      if(get_input_value('_user', RCUBE_INPUT_POST) && get_input_value('_pass', RCUBE_INPUT_POST)){
        $args['action'] = 'login';
      } 
    }
    return $args;  
  }
  
  function switch_lang_after_login($args){
    $lang = get_input_value('_lang_sel', RCUBE_INPUT_POST);
    if(empty($lang))
      return $args;
    if(strtolower($lang) != strtolower($_SESSION['language'])){
      $rcmail = rcmail::get_instance();
      $a_prefs = $rcmail->user->get_prefs();
      $a_prefs['language'] = $lang;
      $a_prefs['auth_time'] = $_SESSION['auth_time'];
      $rcmail->load_language($lang);
      $rcmail->user->save_prefs($a_prefs);   
      $_SESSION['language'] = $lang;
    }
    
    return $args;
  
  }
  
  function add_plugin_localization($lang){
    $rcmail = rcmail::get_instance();
    $langs = $rcmail->config->get('lang_sel_localizations');
    $add = array();
    if(is_array($langs)){
      foreach($langs as $plugin => $folder){       
        if(class_exists($plugin,false)){
          if(! @include "./plugins/" . $plugin ."/" . $folder . "/" . $lang . ".inc")
            @include "./plugins/" . $plugin ."/" . $folder . "/en_US.inc";
          $texts = (array)$labels + (array)$messages + (array)$texts;
          foreach ($texts as $key => $value)
            $add[$plugin.'.'.$key] = $value;
        }
      }
    }
    return $add;
  }
}

?>