<?php

/**
 * Password Reminder
 *
 * @version 2.9.8 - 05.10.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 * @credits Allan Brazute Alves 'EthraZa' http://gplus.to/EthraZa
 *
 **/
 
/**
 *
 * Usage: http://mail4us.net/myroundcube/
 *
 * REQUIREMENTS: plugin savepassword (automatically loaded, do not register the plugin)
 *               plugin settings (automatically loaded, do not register the plugin)
 *               plugin taskbar *
 *               SVN trunk r4025 or higher
 *
 *               (*) If you don't use it, nav in by: ./?_task=settings&_action=plugin.pwtools_reset&_username=[username]
 *                   NOTICE: You can also use a form method post <input type=... name="_username" />
 *
 *
 **/

class pwtools extends rcube_plugin
{
  public $task = 'login|logout|settings';
  public $noajax = true;
  
  /* unified plugin properties */
  static private $plugin = 'pwtools';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = null;
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '2.9.8';
  static private $date = '05-10-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.8.1',
    'PHP' => '5.2.1',
    'required_plugins' => array('settings' => 'require_plugin', 'savepassword' => 'require_plugin'),
    'recommended_plugins' => array('taskbar' => 'config')
  );
  static private $prefs = array('pwtoolsenabled', 'pwtoolsaddress', 'pwtoolsquestion', 'pwtoolsanswer');
  static private $config_dist = 'config.inc.php.dist';
  
  function init()
  {
    $rcmail = rcmail::get_instance();
    $this->require_plugin('savepassword');
    if($_SESSION['username'] == $rcmail->user->data['username'] || $rcmail->task != 'settings'){
      $this->add_texts('localization/');
      $this->_load_config('pwtools');
      $this->include_script('pwtools.js');
    
      // send password message stuff
      $this->add_hook('template_object_pwtoolsresetpw', array($this, 'pwtools_reset_form'));
      $this->add_hook('render_page', array($this, 'add_labels_to_login_page'));
      $this->add_hook('startup', array($this, 'pwtools_reset_do'));
    
      // catch action if requested in not authenticated state
      $this->register_action('plugin.pwtools_reset_do', array($this, 'pwtools_redirect'));
      $this->add_hook('startup', array($this, 'pwtools_reset'));
    
      // catch action if requested in authenticated state
      $this->register_action('plugin.pwtools_reset', array($this, 'pwtools_redirect'));
    
      // summary stuff
      $this->add_hook('template_object_summary_pwtools', array($this, 'summary_html'));
      $this->register_action('plugin.pwtools_show', array($this, 'frame'));
    
      // settings stuff
      $this->add_hook('identity_delete', array($this, 'identity_delete'));
      $this->register_action('plugin.pwtools', array($this, 'pwtools_init'));
      $this->register_handler('plugin.pwtools_form', array($this, 'pwtools_form'));
      $this->register_action('pwtools-save', array($this, 'pwtools_save'));
    }
    
  }
  
  static public function about($keys = false)
  {
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
             $requirements[$prefix.'plugins'][$plugin] = array(
               'method' => $method,
               'plugin' => $plugin,
             );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function identity_delete($p)
  {
    $rcmail = rcmail::get_instance();
    $rc=$rcmail->db->query("
          SELECT * FROM ".get_table_name('identities')." WHERE  identity_id=?", $p['id']
        );
        
    $userrec = $rcmail->db->fetch_assoc($rc);    
    $del = strtolower($userrec['email']);
    $pwa = strtolower($rcmail->config->get('pwtoolsaddress'));

    if(class_exists('hmail_login')){
      if(!strstr($del,'@'))
        $del = $del . "@" . $rcmail->config->get('hmail_default_domain');
      $del = hmail_login::resolve_alias($del);
    }  
    if(is_string($pwa) && is_string($del)){
      $a_prefs = $rcmail->user->get_prefs();
      if(strstr($pwa,$del) && $a_prefs['pwtoolsenabled'] == 1){
        $p['message'] = 'pwtools.nodeletepwtoolsaddress';
        $p['abort'] = true;
      }
      else if(strstr($pwa,$del) && $a_prefs['pwtoolsenabled'] != 1){
        unset($a_prefs['pwtoolsaddress']);
        $rcmail->user->save_prefs($a_prefs);
      }
    }
    return $p;
  }
      
  function _load_config($plugin)
  {
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
      $this->require_plugin('savepassword');
      $this->require_plugin('settings');
    }
  }
  
  // summary stuff
  function frame()
  {
    $rcmail = rcmail::get_instance();
    $rcmail->output->add_script("rcmail.add_onload(\"rcmail.sections_list.select('accountlink')\");");
    $rcmail->output->add_script("rcmail.add_onload(\"document.frames['prefs-frame'].location.href='./?_task=settings&_action=plugin.pwtools&_framed=1'\");");       
    $rcmail->output->send("settings");
    exit;
  }   
  
  function summary_html($args)
  {
    $rcmail = rcmail::get_instance();
    $a_prefs = $rcmail->user->get_prefs();        
    
    $enabled = $a_prefs['pwtoolsenabled'];
    $address = $a_prefs['pwtoolsaddress'];
    
    if($enabled == 1){
      $class = "ok";
      $label_stat = $this->gettext('pwtoolsisenabled');
      $label_stat .= '&nbsp;...&nbsp;(' . $address . ')';
    }
    else{
      $class = "warning";
      $label_stat = $this->gettext('pwtoolsdisabled');      
    }            
    
    $out  = '<fieldset><legend>&nbsp;' . $this->gettext('pwreminder') . '&nbsp;</legend>';
    $out .= '<br /><div>';
    $out .= '&nbsp;' . $label_stat . '&nbsp;...';
    $out .= $label_sens;    
    $out .= '&nbsp;[<a class= "summary' . $class . ' summaryaction" href="./?_task=settings&_action=plugin.pwtools_show">' . $this->gettext('edit') . '</a>]';
    $out .= '</div><br />';
    $out .= '</fieldset>';
    
    $args['content'] = $out;
    return $args;
  }    
  
  // send password message stuff
  
  function pwtools_redirect()
  {
  
    $rcmail = rcmail::get_instance();
    $rcmail->task = "mail";
    $rcmail->action = "";
    header('Location: ./?_task=mail');
    exit;
  }
  
  function add_labels_to_login_page($a){
  
    if($a['template'] != "login" && $a['template'] != 'pwtools.recpw')
      return $a;
  
    $rcmail = rcmail::get_instance();
    $rcmail->output->add_label(
      'pwtools.pwtoolsuserempty',
      'pwtools.pwtoolsusernotfound',
      'pwtools.pwtoolsanswerempty'
    );   
    
    return $a;
  
  }
  
  function pwtools_reset($a){
    if($a['action'] != "plugin.pwtools_reset" || !isset($_SESSION['temp']))
      return $a;

    $rcmail = rcmail::get_instance();

    // kill remember_me and persistent_login cookies
    setcookie ('rememberme_user','',time()-3600);
    setcookie ('rememberme_pass','',time()-3600);
    setcookie ($rcmail->config->get('ifpl_cookie_name', '_pt'),'',time()-3600);
    
    if(!isset($_POST['_username'])){ 
      $user = trim(urldecode($_GET['_username']));
      $host = trim(urldecode($_GET['_host']));
    }
    else{
      $user = get_input_value('_username', RCUBE_INPUT_POST);
      $host = get_input_value('_host', RCUBE_INPUT_POST);
    }
    
    $matchuserhost = $rcmail->config->get('match_user_and_host');
    
    if(class_exists("hmail_login",false)){
      $temparr = explode("@",$user);
      if(!isset($temparr[1]))
        $user = $user . "@" . $rcmail->config->get('hmail_default_domain');
      $user = hmail_login::resolve_alias($user);
    }
    else if($rcmail->config->get('username_domain') !== ''){
      if(!strstr($user,'@')){
        $username_domain = $rcmail->config->get('username_domain');    
        $user = $user . "@" . $username_domain;
      }
    }
    
    if($user != ""){  
      if($matchuserhost){
        if(!$host){
          $host = $rcmail->config->get('default_host', 'localhost');
        }
        $sql_result = $rcmail->db->query("SELECT * FROM ".get_table_name('users')." WHERE  username=? AND mail_host=?", $user, $host);
      }
      else{
        $sql_result = $rcmail->db->query("SELECT * FROM ".get_table_name('users')." WHERE  username=?", $user);
      }

      $userrec = $rcmail->db->fetch_assoc($sql_result);

      // user found
      if(!is_array($userrec))
        $error = 'pwtoolsusernotfound';
      else{
        $prefs = unserialize($userrec['preferences']);   
        //reminder enabled
        if($prefs['pwtoolsenabled'] != 1){
          $error = 'pwtoolsinactive';
        }
        else{
          // secret question/answer and external email defined      
          if($prefs['pwtoolsquestion'] != "" && $prefs['pwtoolsanswer'] != "" && $prefs['pwtoolsaddress'] != ""){
            $_SESSION['pwtools']['username'] = $user;
            $_SESSION['pwtools']['host'] = $userrec['mail_host'];
            $_SESSION['pwtools']['question'] = $prefs['pwtoolsquestion'];
            $_SESSION['pwtools']['answer']   = $prefs['pwtoolsanswer'];
            $_SESSION['pwtools']['email']    = $prefs['pwtoolsaddress'];
            $_SESSION['pwtools']['pass']     = $userrec['password'];
            $rcmail->output->set_pagetitle($this->gettext('pwreminder'));
            $rcmail->config->set('captcha_auth', true);
            $rcmail->output->send("pwtools.recpw");
          }
          else{
            $error = 'pwtoolsimcomplete';
          }
        }
      }
    }
    else{
      $error = 'pwtoolsuserempty';
    }
    
    $rcmail->output->show_message("pwtools.".$error, 'error');
    $rcmail->kill_session();
    $_POST['_user'] = $user;
    $rcmail->output->send('login');
  
  }
  
  function pwtools_reset_form($a){
  
    $user = $_SESSION['pwtools']['username'];
    $host = $_SESSION['pwtools']['host'];
    $question = $_SESSION['pwtools']['question'];
    if(class_exists('hmail_autoban',false)){
      if($_SESSION['pwtools_attempts'] > 6){
        $autoban = new hmail_autoban;
        $autoban->check_autoban(array('user'=>$user, 'localization'=>$this->gettext('autobanned')));
      }
    }    
     
    // return the complete edit form as table
    $display_user = $user;
    if($_SESSION['global_alias'])
      $display_user = $_SESSION['global_alias'];
    $out .= '<fieldset><legend>' . $this->gettext('pwreminder') . ' ::: ' . $display_user . '</legend>' . "\n";
    $out .= '<br />' . "\n";
    $out .= "<table>\n";  
    $field_id = 'pwtoolsquestion';
    $input_pwtoolsquestion = new html_inputfield(array('name' => '_pwtoolsquestion', 'readonly' => 'readonly', 'id' => $field_id, 'value' => $question, 'maxlength' => 320, 'size' => 35, 'autocomplete' => 'off'));

    $out .= sprintf("<tr><td class=\"title\"><label for=\"%s\">%s</label>:</td><td>%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('pwtoolsquestion')),
                $input_pwtoolsquestion->show($question));
                
    $field_id = 'pwtoolsanswer';
    $input_pwtoolsanswer = new html_inputfield(array('name' => '_pwtoolsanswer', 'id' => $field_id, 'value' => '', 'maxlength' => 320, 'size' => 35, 'autocomplete' => 'off'));

    $out .= sprintf("<tr><td class=\"title\"><label for=\"%s\">%s</label>:</td><td>%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('pwtoolsanswer')),
                $input_pwtoolsanswer->show());
                
    $out .= "\n</table>";
    $out .= '<br />' . "\n";
    $out .= "</fieldset>\n";
                       
    $field_id = 'pwtoolsusername';
    $input_pwtoolsusername = new html_hiddenfield(array('name' => '_username', 'id' => $field_id, 'value' => $user));
    $input_pwtoolshost = new html_hiddenfield(array('name' => '_host', 'id' => 'pwtoolshost', 'value' => $host));
    $out .= $input_pwtoolsusername->show($user);
    $out .= $input_pwtoolshost->show($host);
    
    $a['content'] = $out;
    
    return $a;
  
  }
  
  function pwtools_reset_do($a){
      
    if($a['action'] != "plugin.pwtools_reset_do" || !isset($_SESSION['temp']))
      return $a;      
      
    $rcmail = rcmail::get_instance();
    
    $user =  get_input_value('_username', RCUBE_INPUT_POST);
    $temp = explode('@', $user);
    $domain = $temp[count($temp) - 1];
    $host =  get_input_value('_host', RCUBE_INPUT_POST);
    $question = get_input_value('_pwtoolsquestion', RCUBE_INPUT_POST);
    $answer = get_input_value('_pwtoolsanswer', RCUBE_INPUT_POST);
    $pwtoolsanswer = $_SESSION['pwtools']['answer'];
    if($pwtoolsanswer != "")
      $pwtoolsanswer = $rcmail->decrypt($pwtoolsanswer);
      
    if(isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] == TRUE) 
      $s = "s"; 
    $roundcube_url = "http$s://" . $_SERVER['HTTP_HOST'] . $_SERVER['PHP_SELF'] . "?_task=logout&_user=$user&_host=$host"; 
    
    $link  = '<a target="_new" href="' . $roundcube_url . '">'; 
    $link .= "http$s://" . $_SERVER['HTTP_HOST']; 
    $link .= '</a>'; 
  
    if(
      strtolower($question) == strtolower($_SESSION['pwtools']['question']) &&
      strtolower($answer) == strtolower($pwtoolsanswer) &&
      strtolower($user) == strtolower($_SESSION['pwtools']['username']) &&
      isset($_SESSION['pwtools']['email']) &&
      $_SESSION['pwtools']['pass'] != ""
      ){
      
        if(file_exists("./plugins/pwtools/localization/" . $_SESSION['language'] . ".inc")){ 
          $lang = $_SESSION['language']; 
        } 
        else{ 
          $lang = "en_US"; 
        } 
  
        if(file_exists("./plugins/pwtools/localization/" . $lang . "/send_pw_subject.txt")){ 
          $subject_pattern = file_get_contents("./plugins/pwtools/localization/" . $lang . "/send_pw_subject.txt"); 
        } 
        else if(file_exists("./plugins/pwtools/localization/en_US/send_pw_subject.txt")){ 
          $subject_pattern = file_get_contents("./plugins/pwtools/localization/en_US/send_pw_subject.txt"); 
        } 
        else{ 
          $subject_pattern = file_get_contents("./plugins/pwtools/localization/en_US/send_pw_subject.txt.dist"); 
        } 
        
        if(file_exists("./plugins/pwtools/localization/" . $lang . "/send_pw_body.html")){ 
          $body_pattern = file_get_contents("./plugins/pwtools/localization/" . $lang . "/send_pw_body.html"); 
        } 
        else if(file_exists("./plugins/pwtools/localization/en_US/send_pw_body.html")){ 
          $body_pattern = file_get_contents("./plugins/pwtools/localization/en_US/send_pw_body.html"); 
        } 
        else{ 
          $body_pattern = file_get_contents("./plugins/pwtools/localization/en_US/send_pw_body.html.dist"); 
        } 
    
        $footer = $rcmail->config->get('pwtools_footer'); 
        if($body_pattern && $subject_pattern){  
          $subject = sprintf( 
                      $subject_pattern, 
                      $user,
                      $this->getVisitorIP()
                      );
                      
          if (function_exists('mb_encode_mimeheader')){
            mb_internal_encoding(RCMAIL_CHARSET);
            $subject= mb_encode_mimeheader($subject,
              RCMAIL_CHARSET, 'Q', $rcmail->config->header_delimiter(), 8);
          }
          else{
            $subject = '=?UTF-8?B?'.base64_encode($subject). '?=';
          }
                      
          $body = sprintf( 
                      $body_pattern, 
                      $user,
                      $rcmail->decrypt($_SESSION['pwtools']['pass']),
                      $link,
                      $footer,
                      $domain,
                      $rcmail->config->get('admin_email')
                      );
          
          $from = $rcmail->config->get('admin_email');
          $to = $_SESSION['pwtools']['email'];
          
          $ctb = md5(rand() . microtime());

          $headers  = "Return-Path: $from\r\n";
          $headers .= "MIME-Version: 1.0\r\n";
          $headers .= "Content-Type: multipart/alternative; boundary=\"=_$ctb\"\r\n";
          $headers .= "Date: " . date('r', time()) . "\r\n";
          $headers .= "From: $from\r\n";
          $headers .= "To: $to\r\n";
          $headers .= "Subject: $subject\r\n";
          $headers .= "Reply-To: $from\r\n";

          $msg_body .= "Content-Type: multipart/alternative; boundary=\"=_$ctb\"\r\n\r\n";

          $txt_body  = "--=_$ctb";
          $txt_body .= "\r\n";
          $txt_body .= "Content-Transfer-Encoding: 7bit\r\n";
          $txt_body .= "Content-Type: text/plain; charset=" . RCMAIL_CHARSET . "\r\n";
          $LINE_LENGTH = $rcmail->config->get('line_length', 72);  
          $h2t = new html2text($body, false, true, 0);
          $txt = rc_wordwrap($h2t->get_text(), $LINE_LENGTH, "\r\n");
          $txt = wordwrap($txt, 998, "\r\n", true);
          $txt_body .= "$txt\r\n";            
          $txt_body .= "--=_$ctb";
          $txt_body .= "\r\n";
          
          $msg_body .= $txt_body;
          
          $msg_body .= "Content-Transfer-Encoding: quoted-printable\r\n";
          $msg_body .= "Content-Type: text/html; charset=" . RCMAIL_CHARSET . "\r\n\r\n";
          $msg_body .= str_replace("=","=3D",$body);
          $msg_body .= "\r\n\r\n";
          $msg_body .= "--=_$ctb--";
          $msg_body .= "\r\n\r\n";
                
          // send message         
          if (!is_object($rcmail->smtp))
            $rcmail->smtp_init(true);
          
          if($rcmail->config->get('smtp_pass') == "%p"){
            $rcmail->config->set('smtp_server', $rcmail->config->get('default_smtp_server'));
            $rcmail->config->set('smtp_user', $rcmail->config->get('default_smtp_user'));
            $rcmail->config->set('smtp_pass', $rcmail->config->get('default_smtp_pass'));
          }          
          $rcmail->smtp->connect();
          $rc = $rcmail->smtp->send_mail($from, $to, $headers, $msg_body);
          
          if($rc){
            $message = sprintf($this->gettext('pwtools.pwtoolscheckaccount'), $_SESSION['pwtools']['email'] );
            $type = "confirmation";
          }
          else{
            $message = $this->gettext('pwtools.pwtoolssendingfailed');
            $type = "error";
          }
          
          $rcmail->output->command('display_message', $message, $type);
          $rcmail->kill_session();
          $rcmail->output->set_env('keep_alive', false);
          $_POST['_user'] = $user;
          $rcmail->output->send('login');
        
        }
    }
    else{
      if(class_exists('hmail_autoban',false)){
        if(function_exists('curl_init')){
          if(!isset($_SESSION['pwtools_attempts']))
            $_SESSION['pwtools_attempts'] = 0;
          else
            $_SESSION['pwtools_attempts'] = $_SESSION['pwtools_attempts'] + 1;
          if($_SESSION['pwtools_attempts'] > 6){
            $rcmail = rcmail::get_instance();
            if(isset($_SERVER['HTTPS']) && ($_SERVER['HTTPS'] == '1' || strtolower($_SERVER['HTTPS'])=='on'))
              $s = "s";
            else
              $s = "";
            $url = 'http' . $s . '://'.$_SERVER['HTTP_HOST'].$_SERVER['PHP_SELF'] . '?_action=login';
            $ch = curl_init();
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);  
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_FAILONERROR, 1);
            curl_setopt($ch, CURLOPT_TIMEOUT, 3);
            curl_setopt($ch, CURLOPT_POST, 1);
            curl_setopt($ch, CURLOPT_POSTFIELDS, "_user=$user&_pass=" . md5($user) . "&_host=localhost");
            curl_setopt($ch, CURLOPT_COOKIE,"CheckCookie=CheckCookie; domain=.localhost; path=/"); 

            if($rcmail->config->get('http_proxy'))
              curl_setopt($process, CURLOPT_PROXY, $this->proxy . ":" . $rcmail->config->get('http_proxy_port')); 
            $result = curl_exec($ch);

            curl_close($ch);
          }
        }
      }
      // kill keep alive
      $rcmail->output->set_env('keep_alive', false);
      $rcmail->output->show_message('pwtools.pwtoolsfailed','error');
      $rcmail->output->send("pwtools.recpw");
    }
  
  }
  
  function getVisitorIP()
  { 
  
    //Regular expression pattern for a valid IP address 
    $ip_regexp = "/^([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})/"; 

    //Retrieve IP address from which the user is viewing the current page 
    if (isset ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"]) && !empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) { 
      $visitorIP = (!empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) ? $HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"] : ((!empty ($HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'])) ? $HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'] : @ getenv ('HTTP_X_FORWARDED_FOR')); 
    } 
    else { 
      $visitorIP = (!empty ($HTTP_SERVER_VARS['REMOTE_ADDR'])) ? $HTTP_SERVER_VARS['REMOTE_ADDR'] : ((!empty ($HTTP_ENV_VARS['REMOTE_ADDR'])) ? $HTTP_ENV_VARS['REMOTE_ADDR'] : @ getenv ('REMOTE_ADDR')); 
    } 

    return $visitorIP; 
  }
  
  // settings stuff
  
  function pwtools_init()
  {
    
    $rcmail = rcmail::get_instance();
    $rcmail->output->set_pagetitle($this->gettext('pwreminder')); 
    $rcmail->output->send('pwtools.pwtools');
    
  }
  
  function pwtools_save()
  {
  
    $rcmail = rcmail::get_instance();
    $a_prefs = $rcmail->user->get_prefs();  
 
    $pwtools['pwtoolsenabled'] = get_input_value('_pwtoolsenabled', RCUBE_INPUT_POST);
    $pwtools['pwtoolsaddress'] = get_input_value('_pwtoolsaddress', RCUBE_INPUT_POST);
    $pwtools['pwtoolsquestion'] = get_input_value('_pwtoolsquestion', RCUBE_INPUT_POST);
    $pwtools['pwtoolsanswer'] = $rcmail->encrypt(get_input_value('_pwtoolsanswer', RCUBE_INPUT_POST));
    
    $a_prefs = array_merge($a_prefs,$pwtools);
    
    $rcmail->user->save_prefs($a_prefs);
    
    // overwrite object property
    $rcmail->user->data['preferences'] = serialize($a_prefs);
       
    $this->pwtools_init();
  
  }

  function pwtools_form()
  {
    $rcmail = rcmail::get_instance();

    // add some labels to client
    $rcmail->output->add_label(
      'pwtools.pwtoolsquestionempty',
      'pwtools.pwtoolsanswerempty',
      'pwtools.pwtoolsanswernotmatch',
      'pwtools.pwtoolsactive',
      'pwtools.pwtoolsinactive'
    );
    
    $rcmail->output->add_script("var settings_account=true;");  

    $settings = $rcmail->user->get_prefs();
    
    $enabled     = $settings['pwtoolsenabled'];
    $address     = $settings['pwtoolsaddress'];
    $question    = $settings['pwtoolsquestion'];
    $answer      = $rcmail->decrypt($settings['pwtoolsanswer']);

    $IDENTITIES = $rcmail->user->list_identities();

    $identities = array();
    foreach($IDENTITIES as $key => $identity){
      $email = $identity['email'];
      if(class_exists("hmail_login",false))
        $email = hmail_login::resolve_alias($email);
      if(strtolower($email) != strtolower($rcmail->user->data['username']))
        $identities[$email] = strtolower($email);
    }
          
    $rcmail->output->set_env('product_name', $rcmail->config->get('product_name'));

    // allow the following attributes to be added to the <table> tag
    $attrib_str = create_attrib_string($attrib, array('style', 'class', 'id', 'cellpadding', 'cellspacing', 'border', 'summary'));

    // return the complete edit form as table
    $user = $rcmail->user->data['username'];
    if($_SESSION['global_alias'])
      $user = $_SESSION['global_alias'];
    $out .= '<fieldset><legend>' . $this->gettext('pwreminder') . ' ::: ' . $user . '</legend>' . "\n";
    $out .= '<br />' . "\n";
    $out .= '<table' . $attrib_str . ">\n\n";

    // show pwtools properties
    
    if(count($identities) > 0){
      $rcmail->output->add_script("rcmail.pwtools = true;");
      $selector = '<select name="_pwtoolsaddress">' . "\n";
      foreach($identities as $key => $val){
        if(strtolower($address) == $val)
          $selected = " selected=\"selected\"";
        else
          $selected = "";
        $selector .= '<option value="' . $val . "\"$selected>" . $val . "</option>\n";
      }
      $selector .= "</select>\n";
          
      $field_id = 'pwtoolsenabled';
      $input_pwtoolsenabled = new html_checkbox(array('name' => '_pwtoolsenabled', 'id' => $field_id, 'value' => 1));

      $out .= sprintf("<tr><td class=\"title\"><label for=\"%s\">%s</label>:</td><td>%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('pwtoolsenabled')),
                $input_pwtoolsenabled->show($enabled?1:0));
                
      $field_id = 'pwtoolsaddress';

      $out .= sprintf("<tr><td class=\"title\"><label for=\"%s\">%s</label>:</td><td>%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('pwtoolsaddress')),
                $selector);
                
      $field_id = 'pwtoolsquestion';
      $input_pwtoolsquestion = new html_inputfield(array('name' => '_pwtoolsquestion', 'id' => $field_id, 'value' => $question, 'maxlength' => 320, 'size' => 40, 'autocomplete' => 'off'));

      $out .= sprintf("<tr><td class=\"title\"><label for=\"%s\">%s</label>:</td><td>%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('pwtoolsquestion')),
                $input_pwtoolsquestion->show($question));
                
      $field_id = 'pwtoolsanswer';
      $input_pwtoolsanswer = new html_inputfield(array('name' => '_pwtoolsanswer', 'id' => $field_id, 'value' => $answer, 'maxlength' => 320, 'size' => 40, 'autocomplete' => 'off'));

      $out .= sprintf("<tr><td class=\"title\"><label for=\"%s\">%s</label>:</td><td>%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('pwtoolsanswer')),
                $input_pwtoolsanswer->show($answer));
                
      $field_id = 'pwtoolsanswerconfirm';
      $input_pwtoolsanswerconfirm = new html_inputfield(array('name' => '_pwtoolsanswerconfirm', 'id' => $field_id, 'value' => $answerconfirm, 'maxlength' => 320, 'size' => 40, 'autocomplete' => 'off'));

      $out .= sprintf("<tr><td class=\"title\"><label for=\"%s\">%s</label>:</td><td>%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('pwtoolsanswerconfirm')),
                $input_pwtoolsanswerconfirm->show($answer));
    }
    else{
      $target = 'parent';
      $framed = '';
      if($rcmail->config->get('skin', 'classic') == 'larry'){
        $target = 'self';
        $framed = '&_framed=1';
      }
      $rcmail->output->add_script("rcmail.pwtools = false;");
      $field_id = 'pwtoolsaddidentity';
      $out .= sprintf("<tr><td class=\"title\"><label for=\"%s\">%s</label></td><td>%s</td></tr>\n",
                $field_id,
                rep_specialchars_output($this->gettext('pwtoolsaddidentity')),
                "&raquo;&nbsp;<i><a target=\"_$target\" href=\"./?_task=settings&_action=add-identity&_iid=0$framed\">" . $this->gettext('pwtoolsadd') . "</a></i>&nbsp;&laquo;");
    }
                               
    $out .= "\n</table>";
    $out .= '<br />' . "\n";
    $out .= "</fieldset>\n";    

    $rcmail->output->add_gui_object('pwtoolsform', 'pwtools-form');
    
    return $out;
  }

}

?>
