<?php

/**
 * register - self-registration
 *
 * @version 2.7.7 - 23.09.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 **/
 
/**
 *
 * Usage: http://mail4us.net/myroundcube/
 *
 * REQUIREMENTS: plugin taskbar
 *
 *               (*) If you don't use it navigate in by: ./?_task=settings&_action=register
 *
 * Included drivers: #1- hMailserver 5.2.x
 *                   #2- hMailserver 5.3.x
 *                   #3- cPanel
 *                       Thanks to www.justhost.com for setting up a test environment to develop this driver
 *
 * NOTICE: You have to configure the driver (see ./drivers/<driver>/config.inc.php[.dist]
 *
 **/

class register extends rcube_plugin
{

  public $task = 'login|logout|settings';
  public $noframe = true;
  
  /* unified plugin properties */
  static private $plugin = 'register';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = 'NOTE: If you use global_config plugin, you have to configure the driver in global_config\'s configuration.<br />News: cPanel xmlapi driver<br /><a href="http://myroundcube.com/myroundcube-plugins/register-plugin" target="_new">Documentation</a>';
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '2.7.7';
  static private $date = '23-09-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.7.1',
    'hMailserver' => '5.2',
    'PHP' => '5.2.1',
    'recommended_plugins' => array('taskbar' => 'config'),
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';

  function init()
  {
    $this->add_texts('localization/', true);

    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
      $driver = $rcmail->config->get('register_driver');
      if(file_exists("./plugins/register/drivers/$driver/config.inc.php"))
        $this->load_config("drivers/$driver/config.inc.php");
      else
        die("Register plugin ./plugins/register/drivers/$driver/config.inc.php is missing.");
    }
      
    $this->include_script('register.js');
    
    $this->add_hook('startup', array($this, 'register_form'));
    $this->add_hook('template_object_domaingroups', array($this, 'register_domain_groups'));
    $this->add_hook('template_object_domainselector', array($this, 'register_domain_selector'));
    
  }
  
  static public function about($keys = false){
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $plugin,
            );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
      
  function register_form($a){
    if($_GET['_action'] != 'register'){
      return $a;
    }
    
    if(isset($_GET['_ajax'])){
      $this->register_ajax();
    }

    $rcmail = rcmail::get_instance();
    $skin  = $rcmail->config->get('skin');
    
    if(!file_exists("plugins/register/skins/$skin/register.css"))
      $skin = "default";

    $this->include_stylesheet('skins/' . $skin . '/register.css'); 
    $rcmail->output->set_pagetitle($this->gettext('register.sign_up_long'));
    $rcmail->output->add_script("var password_min_length = '" . $rcmail->config->get('password_min_length') ."';");
    if(!$_SESSION['captcha_solved'])
      $rcmail->config->set('captcha_auth', true);
    $rcmail->output->send("register.register");
    exit;
  }
  
  function register_ajax(){
    $rcmail = rcmail::get_instance();
  
    if(
      isset($_POST['_username']) &&
      !empty($_POST['_username']) &&
      isset($_POST['_domain']) &&
      !empty($_POST['_domain']) &&
      isset($_POST['_pass']) &&
      !empty($_POST['_pass'])
    ){
      
      foreach($_POST as $key => $val){
        $_POST[$key] = trim($val);
      }
      
      $account = strtolower($_POST['_username']);
      $disallowed = array_flip($rcmail->config->get('disallowedaccounts'));
      if(isset($disallowed[$account])){
        echo "error:register.disallowedaccount:_username";
        exit;
      }
           
      $user = strtolower($_POST['_username']) . "@" . strtolower($_POST['_domain']);
 
      // include driver
      $rcmail = rcmail::get_instance();
      $driver = $rcmail->config->get('register_driver');
      if(file_exists("./plugins/register/drivers/$driver/driver.inc.php"))
        require_once("./plugins/register/drivers/$driver/driver.inc.php");
      else
        die("Plugin Register: Self Registration Driver not found");
      
      // check if user already exists
      if(rcube_register_user_exists($user)){
        echo "error:register.userexists:_username";
        exit;
      }
      
      // check if alias already exits
      if(rcube_register_alias_exists($user)){
        echo "error:register.userexists:_username";
        exit;
      }
      
      // add user
      $pass = $_POST['_pass'];
      $ret = rcube_register_new_imap_user($user,$pass);
      if($ret === true){
        // pass user to client, register.js will do the login
        if(class_exists('correo')){
           correo::save($this->getVisitorIP());
        }
        echo "ok:$user";
      }
      else if($ret === false){
        echo "error:register.createuserfailed:_username";
      }
      else{
        echo "error:$ret:_username";
      }
      exit;
    }
    else{
      echo "error:register.failure:_username";
    }
    exit;
  }
      
  function register_domain_groups($a){
  
    $rcmail = rcmail::get_instance();
    
    $doms = $rcmail->config->get('webadddoms');
    
    if(count($doms) <= 1)
      return $a;
    
    $sel_group = "";
    if(isset($_GET['_group'])){
      $sel_group = trim(urldecode($_GET['_group']));
    }
    else if(isset($_POST['_group'])){
      $sel_group = trim($_POST['_group']);
    }
    
    $out = "<tr>\n<td class=\"title\" nowrap>" . $this->gettext('register.domain_groups') . "</td>\n<td>\n";
       
    $out .= "\n" . '<select id="sel_group" name="_group" onchange="javascript:register_change_group()">' . "\n";
	
    foreach($doms as $key => $val){
      
      if($sel_group == $key){
        $selected = "selected";
      }
      else{
        $selected = "";
      }
	
      $out .= "\t<option $selected value=\"$key\">$key</option>\n";
    }    
    
    $out .= "</select>\n</td>\n</tr>\n";
  
    $a['content'] = $out;
     
    return $a;
         
  }
  
  function register_domain_selector($a){
  
    $rcmail = rcmail::get_instance();
    $doms = $rcmail->config->get('webadddoms');
    
    if(isset($_GET['_group'])){
      $group = urldecode(trim($_GET['_group']));
    }
    else if(isset($_POST['_group'])){
      $group = trim($_POST['_group']);
    }
    else{
      foreach($doms as $key => $val){
        $group = $key;
        break;
      }
    }

    if(count($doms[$group]) <= 1){
    
      $out = "<input type=\"hidden\" name=\"_domain\" value=\"" . $doms[$group][0] . "\" /><span>" . $doms[$group][0] . "</span>";
    
    }
    else{
    
      $out = "\n" . '<select name="_domain">';
      foreach($doms[$group] as $key => $val){
        $out .= "\t<option value=\"" . $val . "\">" . $val . "</option>\n";
      }
      $out .= "</select>\n";
    }
    
    $a['content'] = $out;
  
    return $a;
    
  }

  function getVisitorIP(){ 
  
    //Regular expression pattern for a valid IP address 
    $ip_regexp = "/^([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})/"; 

    //Retrieve IP address from which the user is viewing the current page 
    if (isset ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"]) && !empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) { 
      $visitorIP = (!empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) ? $HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"] : ((!empty ($HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'])) ? $HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'] : @ getenv ('HTTP_X_FORWARDED_FOR')); 
    } 
    else { 
      $visitorIP = (!empty ($HTTP_SERVER_VARS['REMOTE_ADDR'])) ? $HTTP_SERVER_VARS['REMOTE_ADDR'] : ((!empty ($HTTP_ENV_VARS['REMOTE_ADDR'])) ? $HTTP_ENV_VARS['REMOTE_ADDR'] : @ getenv ('REMOTE_ADDR')); 
    } 

    return $visitorIP; 
  }
}

?>