<?php

/**
 * RSS Feeds 
 *
 * @version 2.5 - 10.09.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 **/
/**
 *
 * Requirements: http_auth plugin (notice this is not equal with http_authentication which is shipped with Roundcube,
 *                                 do not register, plugin is loaded automatically)
 *
 **/
/**
 *
 * Usage: http://mail4us.net/myroundcube
 *
 **/ 
 
class rss_feeds extends rcube_plugin
{
  public $task = "login|logout|mail|settings";
  public $noframe = true;
  public $noajax = true;
  
  /* unified plugin properties */
  static private $plugin = 'rss_feeds';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = null;
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '2.5';
  static private $date = '10-09-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.8',
    'PHP' => '5.2.1',
    'required_plugins' => array(
      'http_auth' => 'require_plugin'
    )
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';
  
  function init(){
    $this->add_texts('localization/');
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
    }
    $this->add_hook('template_object_loginform', array($this, 'show_rss_checkbox'));
    $this->add_hook('login_after', array($this, 'redirect_rss'));
    $this->add_hook('startup', array($this, 'check_auth'));
    $this->require_plugin('http_auth');
    $this->register_action('plugin.rss', array($this, 'show_rss'));
    $this->add_hook('render_page', array($this, 'rss_image'));
  }
  
  static public function about($keys = false){
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
             $requirements[$prefix.'plugins'][$plugin] = array(
               'method' => $method,
               'plugin' => $plugin,
             );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function rss_image($p){
    $rcmail = rcmail::get_instance(); 
    if($p['template'] != "mail" || $rcmail->action == 'plugin.sound_notifier_minimize')
      return $p;
    $skin  = $rcmail->config->get('skin');
    
    // abort if there are no css adjustments
    if(!file_exists('plugins/rss_feeds/skins/' . $skin . '/rss.css')){
      if(!file_exists('plugins/rss_feeds/skins/classic/rss.css'))   
        return $p;
      else
        $skin = "classic";
    }

    $this->include_stylesheet('skins/' . $skin . '/rss.css');
    $this->include_script('rss_feeds.js');
    $html = '<div id="rss"><a href="#" onclick="show_rss_link()">&nbsp;</a></div>';
    $rcmail->output->add_footer($html);
    return $p;  
  }
  
  function check_auth($args){
    if($args['action'] == "rss"){
      header('Location: ./?_task=mail&_action=plugin.rss');
      exit;
    }
    if($args['action'] == "plugin.rss" || $_GET['_action'] == 'show'){
      if(!$_SESSION['user_id']){
        $this->http_auth();
      }
    }
    return $args;
  }

  function http_auth(){

    if(!empty($_POST['_user'])){
      $_SERVER['PHP_AUTH_USER'] = trim($_POST['_user']);
      $_SERVER['PHP_AUTH_PW'] = trim($_POST['_pass']);
    }
  
    if(!isset($_SERVER['PHP_AUTH_USER'])){  
      $this->http_unauthorized();
    }
  }
  
  function http_unauthorized(){
    $rcmail = rcmail::get_instance();
    header('WWW-Authenticate: Basic realm="' . $rcmail->config->get('useragent', 'Webmail') . '"');
    header('HTTP/1.0 401 Unauthorized');
    $js = '
    <html>
    <head>
    <title>' . $rcmail->config->get('useragent', 'Webmail') . '</title>
    <script type="text/javascript">
    <!--
      document.location.href="' . $_SERVER['PHP_SELF'] . '";
    //-->
    </script>
    </head>
    <body></body>
    </html>
    ';
    echo $js;
    exit;
  }
  
  function show_rss_checkbox($args){
    $rcmail = rcmail::get_instance();
    if($rcmail->config->get('show_rss_checkbox')){ 
      $content  = $args['content'];
      $content = str_ireplace ('</tbody>',
        '<tr><td class="title"><label for="rcmrss">' . $this->gettext('rss_plugin_name','rss') . '</label></td><td><input name="_rss" value="1" type="checkbox" /></td>
        </tr></tbody>',$content);
      $args['content'] = $content;
    }
    return $args;
  }
  
  function redirect_rss($args){
    if($_POST['_rss']){
      if(class_exists('wrapper',false)){
        echo '
          <html>
          <head>
          <title>' . $CONFIG['useragent'] . '</title>
          <script type="text/javascript">
          <!--
            parent.location.href="' . $_SERVER['PHP_SELF'] . '?_action=plugin.rss";
          //-->
          </script>
          </head>
          <body></body>
          </html>
        ';
      }
    	else{
        header('Location: ./?_task=mail&_action=plugin.rss');
      }
    	exit;
    }
    return $args;
  }

  function show_rss(){
    $rcmail = rcmail::get_instance();
    $IMAP = $rcmail->imap;
    $CONFIG = $rcmail->config->all();
    include "rss_driver.inc.php";
    exit;
  }
}

?>