<?php
/**
 * sabredav
 *
 * @version 2.5 - 31-10-2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 * 
 **/
 
/**
 *
 * Usage:
 *
 * #1 - Download SabreDAV http://code.google.com/p/sabredav/downloads/list.
 * #2 - Install SabreDAV in the root folder of a (sub)domain.
 * #3 - Create a database named 'sabredav'.
 * #4 - Run SQL-scripts located in SQL folder (Notice: don't use default SabreDAV Scripts).
 * #5 - Copy and paste the files in the folder 'copy_and_paste' into the root of SabreDAV folder.
 * #6 - Edit configuration in index.php (this is one of the files copied into SavreDAV folder in step #5).
 * #7 - Register this plugin either by plugin_manager or in ./config/main.inc.php.
 *
 **/
 
class sabredav extends rcube_plugin
{
  public $task = 'login|settings';
  public $noajax = true;
  
  /* unified plugin properties */
  static private $plugin = 'sabredav';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = '<a href=" http://myroundcube.com/myroundcube-plugins/sabredav-plugin" target="_new">Documentation</a><br />Important Update Notes for Versions 2.x: <a href="http://mirror.mail4us.net/docs/sabredav.html" target="_new">Click here</a>';
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '2.5';
  static private $date = '31-10-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.7.1',
    'PHP' => '5.2.1',
  );
  static private $incompat = null; // null a pair of plugin name and loading method (require_plugin | config)
  static private $sql = array( // null or
    'init' => array( // a pair of datebase driver and the SQL initial script relative to plugin root folder
                'mysql' => 'SQL/mysql.init.sql',
              ),
     'update' => array( // a pair of datebase driver and the SQL update script relative to plugin root folder
                   'mysql' => 'SQL/mysql.update.sql',
                 ),
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';
  
  private $credentials;
  
  function init(){
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $rcmail->config->get('plugins'))){
      $this->load_config();
    }
    $this->add_hook('authenticate', array($this, 'credentials'));
    $this->add_hook('login_after', array($this, 'setup'));
    $this->add_hook('password_change', array($this, 'password'));
  }
  
  static public function about($keys = false){
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $plugin,
            );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function credentials($args){
    if(strtolower($this->get_demo($args['user'])) == strtolower(sprintf(rcmail::get_instance()->config->get('demo_user_account'),""))){
      $args['pass'] = rcmail::get_instance()->config->get('demo_user_password');
    }
    $this->credentials = $args;
  }
  
  function setup($args){
    $rcmail = rcmail::get_instance();
    $this->changepw(strtolower(urldecode($_SESSION['username'])), $rcmail->decrypt($_SESSION['password']));
    $dsn = $rcmail->config->get('db_sabredav_dsn');
    $db = new rcube_mdb2($dsn, '', FALSE);
    $db->set_debug((bool)$rcmail->config->get('sql_debug'));
    $db->db_connect('r');
    
    // create/update users
    $sql_result = $db->query(
      "SELECT * FROM ".$this->table('users').
      " WHERE " . $this->q('username') . "=?",
      $this->credentials['user']);
    $users = array();
    while ($sql_result && ($sql_arr = $db->fetch_assoc($sql_result))) {
      $users[] = $sql_arr;
    }
    if(count($users) > 0){
      $db->query(
        "UPDATE " . $this->table('users') . " 
         SET ".$this->q('digesta1')."=?, ".$this->q('rcube_id')."=?
         WHERE ".$this->q('username')."=?",
        md5($this->credentials['user'] . ':' . $rcmail->config->get('sabredav_realm') . ':' . $this->credentials['pass']),
        $rcmail->user->ID,
        $this->credentials['user']
      );
      $cals = array_merge(array('events'), $rcmail->config->get('sabredav_cals', array()));
      foreach($cals as $idx => $cal){
        $sql_result = $db->query(
          "SELECT * FROM ".$this->table('calendars').
          " WHERE " . $this->q('principaluri') . "=? AND " . $this->q('uri') . "=?",
          "principals/" . $this->credentials['user'],
          $cal
        );
        $calendars = array();
        while ($sql_result && ($sql_arr = $db->fetch_assoc($sql_result))) {
          $calendars[] = $sql_arr;
        }
        if(count($calendars) < 1){
          $db->query(
            "INSERT INTO " . $this->table('calendars') . "
            (".
              $this->q('user_id').", ".
              $this->q('principaluri').", ".
              $this->q('uri').", ".
              $this->q('displayname').", ".
              $this->q('ctag').", ".
              $this->q('calendarorder').", ".
              $this->q('components') . ")
            VALUES (?, ?, ?, ?, ?, ?, ?)",
            $users[0]['id'],
            "principals/" . $this->credentials['user'],
            $cal,
            ucwords($cal),
            0,
            0,
            'VEVENT,VTODO'
          );
        }
      }
      $cards = $rcmail->config->get('sabredav_cards', array());
      foreach($cards as $idx => $card){
        $sql_result = $db->query(
          "SELECT * FROM ".$this->table('addressbooks').
          " WHERE " . $this->q('principaluri') . "=? AND " . $this->q('uri') . "=?",
          "principals/" . $this->credentials['user'],
          $card
        );
        $addressbooks = array();
        while ($sql_result && ($sql_arr = $db->fetch_assoc($sql_result))) {
          $addressbooks[] = $sql_arr;
        }
        if(count($addressbooks) < 1){
          $db->query(
            "INSERT INTO " . $this->table('addressbooks') . "
            (".
              $this->q('user_id').", ".
              $this->q('principaluri').", ".
              $this->q('uri').", ".
              $this->q('displayname').", ".
              $this->q('ctag') . ")
            VALUES (?, ?, ?, ?, ?)",
            $users[0]['id'],
            "principals/" . $this->credentials['user'],
            $card,
            ucwords($card),
            0
          );
        }
      }
    }
    else{
      $db->query(
        "INSERT INTO " . $this->table('users') . "
        (".
          $this->q('username').", ".
          $this->q('digesta1').", ".
          $this->q('rcube_id').")
        VALUES (?, ?, ?)",
        $this->credentials['user'],
        md5($this->credentials['user'] . ':' . $rcmail->config->get('sabredav_realm') . ':' . $this->credentials['pass']),
        $rcmail->user->ID
      );
      
      $userid = $db->insert_id('users_ids');
      
      // create principals
      $sql_result = $db->query(
        "SELECT * FROM ".$this->table('principals').
        " WHERE " . $this->q('uri') . "=?",
        "principals/" . $this->credentials['user']);
      $principals = array();
      while ($sql_result && ($sql_arr = $db->fetch_assoc($sql_result))) {
        $principals[] = $sql_arr;
      }
      if(count($users) < 1){
        $db->query(
          "INSERT INTO " . $this->table('principals') . "
          (".
            $this->q('user_id').", ".
            $this->q('uri').", ".
            $this->q('email').", ".
            $this->q('displayname').")
          VALUES (?, ?, ?, ?)",
          $userid,
          "principals/" . $this->credentials['user'],
          $this->credentials['user'],
          $this->credentials['user']
        );
        $db->query(
          "INSERT INTO " . $this->table('principals') . "
          (".
            $this->q('user_id').", ".
            $this->q('uri').", ".
            $this->q('email').", ".
            $this->q('displayname').")
          VALUES (?, ?, ?, ?)",
          $userid,
          "principals/" . $this->credentials['user'] . "/calendar-proxy-read",
          null,
          null
        );
        $db->query(
          "INSERT INTO " . $this->table('principals') . "
          (".
            $this->q('user_id').", ".
            $this->q('uri').", ".
            $this->q('email').", ".
            $this->q('displayname').")
          VALUES (?, ?, ?, ?)",
          $userid,
          "principals/" . $this->credentials['user'] . "/calendar-proxy-write",
          null,
          null
        );
      }
      
      // create calendars
      $sql_result = $db->query(
        "SELECT * FROM ".$this->table('calendars').
        " WHERE " . $this->q('principaluri') . "=? AND " . $this->q('uri') . "=?",
        "principals/" . $this->credentials['user'],
        "events");
      $calendars = array();
      while ($sql_result && ($sql_arr = $db->fetch_assoc($sql_result))) {
        $calendars[] = $sql_arr;
      }
      if(count($calendars) < 1){
        $cals = array_merge(array('events'), $rcmail->config->get('sabredav_cals', array()));
        foreach($cals as $idx => $cal){
          $db->query(
            "INSERT INTO " . $this->table('calendars') . "
            (".
              $this->q('user_id').", ".
              $this->q('principaluri').", ".
              $this->q('uri').", ".
              $this->q('displayname').", ".
              $this->q('ctag').", ".
              $this->q('calendarorder').", ".
              $this->q('components') . ")
            VALUES (?, ?, ?, ?, ?, ?, ?)",
            $userid,
            "principals/" . $this->credentials['user'],
            $cal,
            ucwords($cal),
            0,
            0,
            'VEVENT,VTODO'
          );
        }
      }
      // create addressbooks
      $cards = $rcmail->config->get('sabredav_cards', array());
      foreach($cards as $idx => $card){
        $sql_result = $db->query(
          "SELECT * FROM ".$this->table('addressbooks').
          " WHERE " . $this->q('principaluri') . "=? AND " . $this->q('uri') . "=?",
          "principals/" . $this->credentials['user'],
          $card
        );
        $addressbooks = array();
        while ($sql_result && ($sql_arr = $db->fetch_assoc($sql_result))) {
          $addressbooks[] = $sql_arr;
        }
        if(count($addressbooks) < 1){
          $db->query(
            "INSERT INTO " . $this->table('addressbooks') . "
            (".
              $this->q('user_id').", ".
              $this->q('principaluri').", ".
              $this->q('uri').", ".
              $this->q('displayname').", ".
              $this->q('ctag') . ")
            VALUES (?, ?, ?, ?, ?)",
            $userid,
            "principals/" . $this->credentials['user'],
            $card,
            ucwords($card),
            0
          );
        }
      }
    }
  }
  
  function create($uri, $type){
    $rcmail = rcmail::get_instance();
    $dsn = $rcmail->config->get('db_sabredav_dsn');
    $db = new rcube_mdb2($dsn, '', FALSE);
    $db->set_debug((bool)$rcmail->config->get('sql_debug'));
    $db->db_connect('r');
    $sql_result = $db->query(
      "SELECT * FROM ".sabredav::table('users').
      " WHERE " . sabredav::q('username') . "=?",
      $rcmail->user->data['username']);
    $users = array();
    while ($sql_result && ($sql_arr = $db->fetch_assoc($sql_result))) {
      $credentials = $sql_arr;
      break;
    }
    if(is_array($credentials)){
      $sql_result = $db->query(
        "SELECT * FROM ".sabredav::table($type).
        " WHERE " . sabredav::q('principaluri') . "=? AND " . sabredav::q('uri') . "=?",
        "principals/" . $credentials['username'],
        $uri
      );
      $result = array();
      while ($sql_result && ($sql_arr = $db->fetch_assoc($sql_result))) {
        $result[] = $sql_arr;
      }
      if(count($result) < 1){
        $db->query(
          "INSERT INTO " . sabredav::table($type) . "
          (".
            sabredav::q('user_id').", ".
            sabredav::q('principaluri').", ".
            sabredav::q('uri').", ".
            sabredav::q('displayname').", ".
            sabredav::q('ctag') . ")
          VALUES (?, ?, ?, ?, ?)",
          $credentials['id'],
          "principals/" . $credentials['username'],
          $uri,
          ucwords($uri),
          0
        );
      }
    }
  }
  
  function get_demo($string){
    $temparr = explode("@",$string);
    return preg_replace ('/[0-9 ]/i', '', $temparr[0]) . "@" . $temparr[count($temparr)-1];   
  }
  
  function password($args){
    sabredav::changepw(strtolower(urldecode($_SESSION['username'])), $args['new_pass']);
  }
  
  static public function changepw($user = '', $password = ''){
    $rcmail = rcmail::get_instance();
    $dsn = $rcmail->config->get('db_sabredav_dsn');
    $db = new rcube_mdb2($dsn, '', FALSE);
    $db->set_debug((bool)$rcmail->config->get('sql_debug'));
    $db->db_connect('r');
    $db->query(
      "UPDATE " . sabredav::table('users') . " 
       SET ". sabredav::q('digesta1')."=?
       WHERE ". sabredav::q('username')."=?",
      md5($user . ':' . $rcmail->config->get('sabredav_realm') . ':' . $password),
      $user
    );
  }
  
  static public function q($str){
    return rcmail::get_instance()->db->quoteIdentifier($str);
  }
  
  static public function table($str){
    return rcmail::get_instance()->db->quoteIdentifier(get_table_name($str));
  }
}
?>