<?php
/**
 * scheduled_sending
 *
 * @version 1.9 - 31.07.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 * 
 **/
 
/**
 *
 * Usage: http://mail4us.net/myroundcube/
 *
 * Requirements: (*) Roundcube v0.8
 *               (*) jquerui plugin (do not register, plugin is loaded automatically)
 *
 * Notice:
 *
 * cron job / windows task to send scheduled messages (suggested interval 5 mins)
 *       <path_to_php.exe>/php.exe -f "<path_to_roundcube>/plugins/scheduled_sending/cron/sendmail.php"
 *
 **/
 
class scheduled_sending extends rcube_plugin
{
  private $rcmail;
  private $skin_path;
  public $task = 'login|logout|mail|settings';
  
  /* unified plugin properties */
  static private $plugin = 'scheduled_sending';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = null;
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '1.9';
  static private $date = '31-07-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.8',
    'PHP' => '5.2.1',
    'required_plugins' => array('jqueryui' => 'require_plugin'),
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';
  
  function init(){
    $this->rcmail = rcmail::get_instance();
    if(!in_array('global_config', $this->rcmail->config->get('plugins'))){
      $this->load_config('config/config.inc.php');
    }
    
    $this->skin_path = $this->local_skin_path();
    $this->register_action('plugin.scheduled_sending_remove', array($this, 'remove'));
    $this->register_action('plugin.scheduled_sending_uninstall', array($this, 'uninstall'));
    $this->add_hook('message_outgoing_headers', array($this, 'message_outgoing_headers'));
    $this->add_hook('messages_list', array($this, 'pass_scheduled_header'));
    $this->add_hook('message_compose', array($this, 'save_scheduled_sending_date'));
    $this->add_hook('message_compose_body', array($this, 'launch_jqueryui_dialog'));
    $this->add_hook('storage_init', array($this, 'fetch_ics_header'));
    $this->add_hook('message_draftsaved', array($this, 'draftsaved'));
    $this->add_hook('login_after', array($this, 'sync_imap'));
    $this->add_hook('messages_list', array($this, 'sync_imap'));
    
    if(
        $this->rcmail->action == 'compose' ||
        $this->rcmail->action == 'preview' ||
        $this->rcmail->action == 'show' ||
        !$this->rcmail->action
      ){
      $this->add_texts('localization/', true);
      $this->include_stylesheet($this->skin_path . '/scheduled_sending.css');
      $this->rcmail->output->set_env('scheduled_sending_skin', $this->skin_path);
      $this->include_script('scheduled_sending.js');
    }
    if($this->rcmail->action == 'compose'){
      $this->scheduled_sending_init();
    }
  }

  static public function about($keys = false){
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $plugin,
            );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function uninstall(){
    if(!empty($this->rcmail->user->ID)){
      $query = $this->rcmail->db->query(
        "DELETE FROM " . $this->table('scheduled_sending') . " WHERE " . $this->q('user_id') . "=?",
        $this->rcmail->user->ID
      );
    }
    $this->rcmail->output->command('plugin.plugin_manager_success', '');
  }
  
  function scheduled_sending_init(){
    $this->require_plugin('jqueryui');
    $this->include_stylesheet($this->skin_path . '/jquery.timepicker.css');
    $this->include_script('jquery.timepicker.js');
    $this->rcmail->output->add_footer(
      html::div(array('id' => 'scheduled_sending_dialog', 'style' => 'display:none;'),
        html::div(array('id' => 'scheduled_sending_datetimepicker', 'align' => 'center')))
    );
    $this->rcmail->output->set_env('rc_time_format', $this->rcmail->config->get('time_format', 'H:i'));
    $this->add_button(array(
      'command' => 'plugin.scheduled_sending_dialog',
      'onclick' => 'scheduled_sending_datepicker()',
      'type' => 'link',
      'label' => 'scheduled_sending.buttontitle',
      'class' => 'button scheduled_sending',
      'title' => 'scheduled_sending.buttontitle',
      'id' => 'rcmbtn_scheduled_sending'), 'toolbar');
    // datetimepicker bug ... init datetimepicker before TinyMCE
    $script = 'if(typeof scheduled_sending_datepicker == "function"){' . "\n" .
              '  scheduled_sending_datepicker();' . "\n" .
              '  $("#scheduled_sending_dialog").dialog("close");' . "\n" .
              '}';
    $this->rcmail->output->add_script($script, 'docready');
  }
  
  function sync_imap($a=array()){
    if(!empty($this->rcmail->user->ID)){
      $result = $this->rcmail->db->query("SELECT " . $this->q('uid') . ", " . $this->q('flag') . ", " . $this->q('target') . " FROM " . $this->table('scheduled_sending') . "
                                          WHERE " . $this->q('user_id') . "=? AND " . $this->q('flag') . "=?",
                                          $this->rcmail->user->ID,
                                          1
                                        );
      $messages = array();
      while ($result && ($message = $this->rcmail->db->fetch_assoc($result))) {
        $messages[] = $message;
      }
      $imap = $this->rcmail->imap;
      foreach($messages as $message){
        $moved = $imap->move_message($message['uid'], $message['target'], $this->rcmail->config->get('drafts_mbox'));
        if($moved){
          $query = $this->rcmail->db->query(
            "DELETE FROM " . $this->table('scheduled_sending') . " WHERE " . $this->q('uid') . "=? AND " . $this->q('user_id') . "=?",
            $message['uid'],
            $this->rcmail->user->ID
          );
        }
        else{
          // find me: Error handling ...
          // detect flag=2 and force user to send messages manually
        }
      }
    }
    return $a;
  }
  
  function draftsaved($a){
    if($scheduled = get_input_value('_scheduled', RCUBE_INPUT_GPC)){
      if(!empty($this->rcmail->user->ID)){
        $imap = $this->rcmail->imap;
        $imap->set_mailbox($a['folder']);
        $raw = $imap->get_raw_body($a['uid']);
        $temparr = explode("\n", $raw);
        //preg_match('#Subject: (.*?)$#im', $headers, $matches); $matches[1]
        foreach($temparr as $key => $val){
          if(strtolower(substr($val,0,strlen('bcc:'))) == 'bcc:'){
            unset($temparr[$key]);
            break;
          }
        }
        foreach($temparr as $key => $val){
          if(strtolower(substr($val,0,strlen('x-rc-scheduled:'))) == 'x-rc-scheduled:'){
            unset($temparr[$key]);
            break;
          }
        }
        $raw = implode("\n",$temparr);
        $target =  get_input_value('_store_target', RCUBE_INPUT_GPC);
        $input_charset = $this->rcmail->output->get_charset();
        $message_charset = isset($_POST['_charset']) ? $_POST['_charset'] : $input_charset;
        $to = rcmail_email_input_format(get_input_value('_to', RCUBE_INPUT_POST, TRUE, $message_charset), true);
        $cc = rcmail_email_input_format(get_input_value('_cc', RCUBE_INPUT_POST, TRUE, $message_charset), true);
        $bcc = rcmail_email_input_format(get_input_value('_bcc', RCUBE_INPUT_POST, TRUE, $message_charset), true);
        $subject = get_input_value('_subject', RCUBE_INPUT_GPC);
        $result = $this->rcmail->db->query("SELECT " . $this->q('uid') . " FROM " . $this->table('scheduled_sending') . "
                                        WHERE " . $this->q('user_id') . "=?",
                                        $this->rcmail->user->ID
                                      );
        $messages = array();
        while ($result && ($message = $this->rcmail->db->fetch_assoc($result))) {
          $messages[] = $message;
        }
        foreach($messages as $message){
          if(!$imap->get_raw_headers($message['uid'])){
            $query = $this->rcmail->db->query(
              "DELETE FROM " . $this->table('scheduled_sending') . " WHERE " . $this->q('uid') . "=? AND " . $this->q('user_id') . "=?",
              $message['uid'],
              $this->rcmail->user->ID
            );
          }
        }
        $query = $this->rcmail->db->query(
          "INSERT INTO " . $this->table('scheduled_sending') . "
          (" . $this->q('user_id') . ", " .
               $this->q('uid') . ", " .
               $this->q('mailbox') . ", " .
               $this->q('target') . ", " .
               $this->q('to') . ", " .
               $this->q('cc') . ", " .
               $this->q('bcc') . ", " .
               $this->q('subject') . ", " .
               $this->q('raw') . ", " .
               $this->q('runtime') . ", " .
               $this->q('flag') . ")
          VALUES (?, ?, ?, ?, ?, ?, ?, ?, ?, ?, ?)",
          $this->rcmail->user->ID,
          $a['uid'],
          $a['folder'],
          $target,
          $to,
          $cc,
          $bcc,
          $subject,
          $raw,
          $scheduled,
          0
        );
        $this->rcmail->output->command('plugin.scheduled_sending_savedraft_callback', array('uid' => $a['uid']));
      }
    }
    return $a;
  }
  
  function remove(){
    if($uid = get_input_value('_uid', RCUBE_INPUT_GPC)){
      if(!empty($this->rcmail->user->ID)){
        $query = $this->rcmail->db->query(
          "DELETE FROM " . $this->table('scheduled_sending') . " WHERE " . $this->q('uid') . "=? AND " . $this->q('user_id') . "=?",
          $uid,
          $this->rcmail->user->ID
        );
      }
    }
    $imap = $this->rcmail->imap;
    if($this->rcmail->config->get('flag_for_deletion')){
      $marked = $imap->set_flag($uid, 'deleted');
      if($marked){
         $this->rcmail->output->show_message('messagemarked', 'confirmation');
      }
      else{
        rcmail_display_server_error('errormarking');
        $this->rcmail->output->send();
      }
    }
    else{
      $moved = $imap->move_message($uid, $this->rcmail->config->get('trash_mbox'), $this->rcmail->config->get('drafts_mbox'));
      if($moved){
        $this->rcmail->output->show_message('messagemoved', 'confirmation');
      }
      else{
        rcmail_display_server_error('errormoving');
        $this->rcmail->output->send();
       }
    }
    $this->rcmail->output->command('plugin.scheduled_sending_savedraft_callback', array('removed' => true, 'uid' => $uid));
  }
  
  function launch_jqueryui_dialog($a){
    if($_SESSION['scheduled_sending_date']){
      $this->rcmail->output->set_env('scheduled_sending_ts', $_SESSION['scheduled_sending_date'] * 1000);
      $this->rcmail->output->set_env('scheduled_sending_uid', $_SESSION['scheduled_sending_uid']);
      $this->rcmail->session->remove('scheduled_sending_date');
    }
    return $a;
  }
  
  function save_scheduled_sending_date($a){
    if($date = get_input_value('_date', RCUBE_INPUT_GPC)){
      $_SESSION['scheduled_sending_date'] = $date;
    }
    if($uid = get_input_value('_uid', RCUBE_INPUT_GPC)){
      $_SESSION['scheduled_sending_uid'] = $uid;
    }
    return $a;
  }
  
  function fetch_ics_header($p){
    $add_headers = array('User-Agent','X-RC-Scheduled');
    $p['fetch_headers'] = trim($p['fetch_headers'].' ' . strtoupper(join(' ', $add_headers)));
    return $p;
  }
  
  function pass_scheduled_header($p){
    if(is_array($p['messages'])){
      foreach($p['messages'] as $message){
        if($message->others['x-rc-scheduled']){
          $message->list_flags['extra_flags']['scheduled'] = $message->others['x-rc-scheduled'];
        }
      }
    }
    return $p;
  }
  
  function message_outgoing_headers($a){
    if($scheduled = get_input_value('_scheduled', RCUBE_INPUT_GPC)){
      $a['headers']['Date'] = date('r', $scheduled);
      $a['headers']['X-RC-Scheduled'] = $scheduled;
    }
    else{
      if(get_input_value('_draft', RCUBE_INPUT_GPC)){
        $a['headers']['Date'] = date('r', time());
        if($uid = get_input_value('_uid', RCUBE_INPUT_GPC)){
          $query = $this->rcmail->db->query(
            "DELETE FROM " . $this->table('scheduled_sending') . " WHERE " . $this->q('uid') . "=? AND " . $this->q('user_id') . "=?",
            $uid,
            $this->rcmail->user->ID
          );
        }
      }
      unset($a['headers']['X-RC-Scheduled']);
    }
    return $a;
  }
  
  private function q($str){
    return $this->rcmail->db->quoteIdentifier($str);
  }
  
  private function table($str){
    return $this->rcmail->db->quoteIdentifier(get_table_name($str));
  }
}
?>