<?php

/**
 * Terms and Conditions
 *
 * @version 2.9.1 - 10.09.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 **/
 
/**
 *
 * Usage: http://mail4us.net/myroundcube/
 *
 * Notice: This plugin must run before all other plugins which are using
 *         the 'login_after' hook !!!
 *
 **/
 
class terms extends rcube_plugin
{
  public $task = '?(?!login|logout).*';
  public $noajax = true;
  public $noframe = true;
  
  /* unified plugin properties */
  static private $plugin = 'terms';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = null;
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '2.9.1';
  static private $date = '10-09-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.8.1',
    'PHP' => '5.2.1'
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';
  
  function init()
  {
    $this->add_hook('render_page', array($this, 'terms_check'));
    $this->register_action('plugin.terms', array($this, 'terms_submitted'));
    $this->register_action('plugin.terms_show', array($this, 'terms_show'));
  }
  
  static public function about($keys = false){
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $plugin,
            );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }
  
  function terms_check($a){
    if(!isset($_POST['terms_agreed']) && empty($_SESSION['temp']) && $a['template'] != 'terms.terms' && empty($_SESSION['terms']) && rcmail::get_instance()->action != "plugin.plugin_manager_update"){
      $a = $this->terms_show($a);
    }
    return $a;
  }

  function terms_show($args)
  {
    $rcmail = rcmail::get_instance();
    if($_SESSION['demouser'] && $rcmail->config->get('demo_target_group', 'admins') != 'admins'){
      $_SESSION['terms'] = true;
      return $args;
    }
    if(!in_array('global_config', $rcmail->config->get('plugins', array()))){
      $this->load_config();
    }
    $this->add_texts('localization/', true);
    $renew = (int)strtotime($rcmail->config->get('terms_date','2011-01-01 00:00:00'));
    $auto = $rcmail->config->get('terms_renew');
    $old = $rcmail->config->get('termsagreed',0);
    if(
        $renew > $old || // terms have been updated
        (time() - ($auto * 86400) > $old && $auto != 0) || // renew agreement after x days
        $rcmail->config->get('terms_always',false) // always renew agreement
      ){
      $this->terms_css();
      if(file_exists('./plugins/terms/localization/' . $_SESSION['language'] . '_terms.html'))
        $_SESSION['terms_lang'] = $_SESSION['language'];
      else
        $_SESSION['terms_lang'] = 'en_US';
      if(!file_exists('./plugins/terms/localization/' . $_SESSION['terms_lang'] . '_terms.html')){
         die('Misconfiguration: ./plugins/terms/localization/' . $_SESSION['terms_lang'] . '_terms.html does not exist.');
      }
      $rcmail->output->set_env("action", "");
      $rcmail->output->set_env("template", "terms");
      $rcmail->output->send("terms.terms");
    }
    else{
      $_SESSION['terms'] = true;
    }
    return $args;
  }
  
  function terms_submitted()
  {
    $rcmail = rcmail::get_instance();
    
    if($rcmail->task == "logout")
      return $a;
    
    if(isset($_POST['terms_agreed'])){
      $_SESSION['terms'] = true;
      $settings = (array)unserialize($rcmail->user->data['preferences']);
      $settings = array_merge($settings,array('termsagreed' => time()));    
      $rcmail->user->save_prefs($settings);
      $this->log(sprintf("TERMS ACCEPTED: %s", $this->getVisitorIP() . ": " . $_SESSION['username']));
      $rcmail->output->redirect(array('task' => 'mail', 'action' => ''));
    }
    else{
      $rcmail->session->remove('terms');
      $this->terms_css();
      $rcmail->output->show_message('terms.termsmustagree','error');
      $rcmail->kill_session();
      $rcmail->output->send('terms.logout');
      exit;
    }
    return;  
  }
  
  function terms_css()
  {
    $rcmail = rcmail::get_instance();    
    $skin  = $rcmail->config->get('skin','classic');
    if(!file_exists("plugins/terms/skins/$skin/terms.css"))
      $skin = "classic";
    $this->include_stylesheet('skins/' . $skin . '/terms.css');
  }
  
  function getVisitorIP()
  { 
  
    //Regular expression pattern for a valid IP address 
    $ip_regexp = "/^([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})\.([0-9]{1,3})/"; 

    //Retrieve IP address from which the user is viewing the current page 
    if (isset ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"]) && !empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) { 
      $visitorIP = (!empty ($HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"])) ? $HTTP_SERVER_VARS["HTTP_X_FORWARDED_FOR"] : ((!empty ($HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'])) ? $HTTP_ENV_VARS['HTTP_X_FORWARDED_FOR'] : @ getenv ('HTTP_X_FORWARDED_FOR')); 
    } 
    else { 
      $visitorIP = (!empty ($HTTP_SERVER_VARS['REMOTE_ADDR'])) ? $HTTP_SERVER_VARS['REMOTE_ADDR'] : ((!empty ($HTTP_ENV_VARS['REMOTE_ADDR'])) ? $HTTP_ENV_VARS['REMOTE_ADDR'] : @ getenv ('REMOTE_ADDR')); 
    } 

    return $visitorIP; 
  }
  
  function log($log)
  { 
    $rcmail = rcmail::get_instance();
    if($rcmail->config->get('terms_log',true)){
      write_log('terms',$log);
    }  
  }
}
?>