<?php

/**
 * Wrapper
 * - Frame RoundCube and display and display f.e. ads in left and/or right frame
 *
 * @version 3.1.2 - 10.09.2012
 * @author Roland 'rosali' Liebl
 * @website http://myroundcube.googlecode.com
 * @licence GNU GPL
 *
 **/
 
/**
 *
 * Usage: http://mail4us.net/myroundcube/
 *
 **/
 
class wrapper extends rcube_plugin
{
  public $noajax = true;

  /* unified plugin properties */
  static private $plugin = 'wrapper';
  static private $author = 'myroundcube@mail4us.net';
  static private $authors_comments = null;
  static private $download = 'http://myroundcube.googlecode.com';
  static private $version = '3.1.2';
  static private $date = '10-09-2012';
  static private $licence = 'GPL';
  static private $requirements = array(
    'Roundcube' => '0.8.1',
    'PHP' => '5.2.1',
  );
  static private $prefs = null;
  static private $config_dist = 'config.inc.php.dist';
  
  private $templates = array(
                                'login',
                                'summary.summary',
                                'addressbook',
                                'settings',
                                'calendar.calendar',
                                'sticky_notes.sticky_notes',
                                'planner.planner',
                              );
  
  function init()
  {
    $rcmail = rcmail::get_instance();
    if(!in_array('global_config', $plugins = $rcmail->config->get('plugins'))){
      $this->load_config();
    }
    $this->register_action('plugin.wrapper_left', array($this, 'wrapper_left'));
    $this->register_action('plugin.wrapper_right', array($this, 'wrapper_right'));
      
    $this->add_hook('render_page', array($this, 'check_frame'));
    //$this->add_hook('template_object_loginform', array($this, 'users_width'));
    //$this->add_hook('template_object_sectionslist', array($this, 'check_frame'));
    $this->add_hook('startup', array($this, 'load_frame'));
    $this->add_hook('template_object_check_frame', array($this, 'users_width'));
    $this->add_hook('render_page', array($this, 'sync_title'));
    if ($rcmail->task == 'settings') {
      $dont_override = $rcmail->config->get('dont_override', array());
      if (!in_array('wrapper_width', $dont_override)) {
        $this->add_hook('preferences_list', array($this, 'prefs_table'));
        $this->add_hook('preferences_save', array($this, 'save_prefs'));
      }
    }
  }
  
  static public function about($keys = false)
  {
    $requirements = self::$requirements;
    foreach(array('required_', 'recommended_') as $prefix){
      if(is_array($requirements[$prefix.'plugins'])){
        foreach($requirements[$prefix.'plugins'] as $plugin => $method){
          if(class_exists($plugin) && method_exists($plugin, 'about')){
            /* PHP 5.2.x workaround for $plugin::about() */
            $class = new $plugin(false);
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $class->about($keys),
            );
          }
          else{
            $requirements[$prefix.'plugins'][$plugin] = array(
              'method' => $method,
              'plugin' => $plugin,
            );
          }
        }
      }
    }
    $rcmail_config = array();
    if(is_string(self::$config_dist)){
      if(is_file($file = INSTALL_PATH . 'plugins/' . self::$plugin . '/' . self::$config_dist))
        include $file;
      else
        write_log('errors', self::$plugin . ': ' . self::$config_dist . ' is missing!');
    }
    $ret = array(
      'plugin' => self::$plugin,
      'version' => self::$version,
      'date' => self::$date,
      'author' => self::$author,
      'comments' => self::$authors_comments,
      'licence' => self::$licence,
      'download' => self::$download,
      'requirements' => $requirements,
    );
    if(is_array(self::$prefs))
      $ret['config'] = array_merge($rcmail_config, array_flip(self::$prefs));
    else
      $ret['config'] = $rcmail_config;
    if(is_array($keys)){
      $return = array('plugin' => self::$plugin);
      foreach($keys as $key){
        $return[$key] = $ret[$key];
      }
      return $return;
    }
    else{
      return $ret;
    }
  }

  function users_width($args){
    $rcmail = rcmail::get_instance();
    if($rcmail->task == 'login' || $rcmail->task == 'logout'){
      $width = $rcmail->config->get('wrapper_width_default',1024);
    }
    else{
      $width = $rcmail->config->get('wrapper_width', $rcmail->config->get('wrapper_width_default',1024));
    }
    if($rcmail->config->get('wrapper_default_width',1024) != $width){
      $script  = 'try{parent.parent.document.getElementById("wrapper").cols = "*,' . $width . ',*"}catch(e){};';
      $script .= 'try{parent.parent.child_task =  rcmail.env.task; parent.parent.doResize(' . $width . ')}catch(e){};';
      $rcmail->output->add_script($script);
    }
    return $args;
  }
  
  function prefs_table($args)
  {
    if ($args['section'] == 'general') {
      $this->add_texts('localization');
      $rcmail = rcmail::get_instance();
      $formats = $rcmail->config->get('wrapper_inner_widths',array(900, 1000, 1100, 1200, 1300, 1400, 1500, 1600));
      $field_id = 'rcmfd_warpperwidth';
      $select = new html_select(array('name' => '_wrapper_width', 'id' => $field_id));
      $select->add($formats);
      $args['blocks']['main']['options']['wrapperwidth']['title'] = Q($this->gettext('wrapper.width'));
      $args['blocks']['main']['options']['wrapperwidth']['content'] = $select->show((int)$rcmail->config->get('wrapper_width',1024), array('name' => "_wrapper_width"));
    }
    return $args;
  }

  function save_prefs($args)
  {
    if($args['section'] == 'general'){
      $rcmail = rcmail::get_instance();
      $width = get_input_value('_wrapper_width', RCUBE_INPUT_POST);
      $old_width = $rcmail->config->get('wrapper_width',1024);
      if($old_width != $width){
        $rcmail->output->add_script('parent.parent.document.getElementById("wrapper").cols = "*,' . $width . ',*";');
        $rcmail->output->add_script('parent.parent.child_task =  rcmail.env.task; parent.parent.doResize(' . $width . ')');
      }    
      $args['prefs']['wrapper_width'] = $width;
      return $args;
    }
  }
  
  function check_frame($args)
  {
    if(isset($_GET['_framed'])){
      return $args;
    }
    else if(preg_match("/mozilla/i", $_SERVER['HTTP_USER_AGENT'])){
      // common browsers
    }
    else if(preg_match("/opera/i", $_SERVER['HTTP_USER_AGENT'])){
      // common browsers
    }    
    else{
      // assume crawler
      return $args;
    }
    
    if(in_array($args['template'], $this->templates)){
      $rcmail = rcmail::get_instance();
      $script = 'function maxWidth() { w = $(document).width(); return w; }' . "\n";
      $rcmail->output->add_script($script);
      $skin = get_input_value('skin', RCUBE_INPUT_GPC);
      if(!empty($skin)){
        $skin="&skin=$skin";
      }
      $script = 'if(!parent.roundcube_wrapper || parent.roundcube_skin != "' . $rcmail->config->get('skin','classic') . '"){parent.location.href="./?_redirect=' . $rcmail->action . '&_task=' . $rcmail->task . '&_action=plugin.wrapper&_screen_width=" + maxWidth() + "&_default_width=' . $rcmail->config->get('wrapper_width_default',1024) . $skin . '"};';
      $rcmail->output->add_script("try{" . $script . "}catch(e){};","foot");
    }
    return $args;
  }
  
  function load_frame($args)
  {
    $rcmail = rcmail::get_instance();  
    if($args['action'] == 'plugin.wrapper'){
      $action = get_input_value('_redirect', RCUBE_INPUT_GET);
      $left_width = '*';
      $right_width = '*';
      $content_width = (int)$rcmail->config->get('wrapper_width', $rcmail->config->get('wrapper_width_default',1024));
      $left_space = 0;
      if(!empty($_GET['_screen_width'])){
        $screen_width = (int)$_GET['_screen_width'];
        $left_space = round(($screen_width - $content_width) / 2,0);
      }
      else{
        $rcmail->output->send('login');
      }
      $listener = $rcmail->config->get('wrapper_resize_listener',true);
      $frameset = 
      '<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.0 Transitional//EN" "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd">
      <html xmlns="http://www.w3.org/1999/xhtml">
      <head>
      <title>' . str_replace('$product', $rcmail->config->get('product_name'), $this->gettext('welcome')) . '</title>
      <link rel="shortcut icon" href="skins/' . $rcmail->config->get('skin','classic') . '/images/favicon.ico"/>
      <script type="text/javascript" src="program/js/jquery.min.js"></script>
      <script type="text/javascript">
      <!--
        var roundcube_skin = "' . $rcmail->config->get('skin','classic') . '";
        var roundcube_wrapper = true;';
        if($listener){
          $frameset .=
        '
        function maxWidth() {
          w = $(window).width();
          return w; 
        }
        var current_left = ' . $left_space . ';
        var child_task = "login";
        function doResize(content){
          if(child_task != "login"){
            if(!content) content = ' . $content_width . ';
            var max = maxWidth();
            var left_space = Math.round((max-content)/2);
            if(left_space != current_left){
              window.frames[0].location.href = "./?_action=plugin.wrapper_left&_space=" + left_space;
              window.frames[2].location.href = "./?_action=plugin.wrapper_right&_space=" + left_space;
              current_left = left_space;
            }
          }
        }
        window.onresize = doResize;';
      }
      $frameset .= 
      '
        function sync_title(){
          setTimeout("sync_title()", 5000);
          document.title = window.frames[1].document.title;
        }
      //-->
      </script>
      </head>
      <frameset onload="sync_title()" id="wrapper" cols="' . $left_width . ',' . $content_width . ',' . $right_width . '" frameborder="0" framespacing="0" border="0">
      <frame scrolling="no" name="left" id="left" src="' . $rcmail->config->get('wrapper_left','./?_action=plugin.wrapper_left') . '&_space=' . $left_space . '" />
      <frame scrolling="no" name="wrapper" id="content" src="./?_task=' . $rcmail->task . '&_action=' . $action . '" />
      <frame scrolling="no" name="right" id="right" src="' . $rcmail->config->get('wrapper_right','./?_action=plugin.wrapper_right') . '&_space=' . $left_space . '" />
      </frameset>
      <noframes>
        <body> Your browser does not support frames ! </body>
      </noframes>
      </html>';
      echo $frameset;
      exit;
    }
    $path = "./plugins/wrapper/skins/" . $rcmail->config->get('skin','classic') . "/templates/";
    if(!is_dir($path)){
      $path = "./plugins/wrapper/skins/classic/templates/";
    }
    if($args['action'] == 'plugin.wrapper_left'){
      $min_width = $rcmail->config->get('wrapper_left_width_min',120);
      $width = (int)$_GET['_space'];
      if($width < $min_width)
        $rcmail->config->set('wrapper_left_template','left');
      echo file_get_contents($path .  $rcmail->config->get('wrapper_left_template','left') . ".html");
      exit;
    }
    if($args['action'] == 'plugin.wrapper_right'){
      $min_width = $rcmail->config->get('wrapper_right_width_min',120);
      $width = (int)$_GET['_space'];
      if($width < $min_width)
        $rcmail->config->set('wrapper_right_template','right');
      echo file_get_contents($path .  $rcmail->config->get('wrapper_right_template','right') . ".html");
      exit;
    }
  }
  
  function wrapper_left()
  {
    return;
  }
  
  function wrapper_right()
  {
    return;
  }
 
  function sync_title($args)
  {
    $rcmail = rcmail::get_instance();
    $script = 'rcmail.add_onload(\'if(parent.roundcube_wrapper){parent.document.title=document.title};\')';
    $rcmail->output->add_script($script, 'docready');
    return $args;
  }
}

?>